<?php

/**
 * Class BonAchat
 *
 * @property $bon_achat_id
 * @property $bon_achat_prix
 */
class BonAchat extends Model
{

    const BA_UTILISE = 0;
    const BA_VALIDE = 1;
    const BA_JOUR_DEFAULT = 30;

    protected $table = 'bon_achat';

    /**
     * permet de récuperer une liste de bons d'achat
     *
     * @param array $formvars
     * @param string $champs
     *
     * @return mixed
     */
    public function get($formvars = array(), $champs = "")
    {

        $sql = "SELECT *, CONCAT('BA',bon_achat_id) AS ba_id";
        $sql.=" FROM (bon_achat ba)";
        $sql.=" WHERE 1";

        if (!empty($formvars['client_id']))
            $sql.=" AND client_id = '" . $formvars['client_id'] . "'";

        if (!empty($formvars['bon_achat_id']))
            $sql.=" AND bon_achat_id = '" . $formvars['bon_achat_id'] . "'";

        if (!empty($formvars['bon_achat_statut_id']))
            $sql.=" AND bon_achat_statut_id = '" . $formvars['bon_achat_statut_id'] . "'";

        if (!empty($formvars['interval'])) {
            if (!empty($formvars['interval'][0])) {
                $sql .= " AND bon_achat_date_limit >= '" . $formvars['interval'][0] . "'";
            }

            if (!empty($formvars['interval'][1])) {
                $sql .= " AND bon_achat_date_limit <= '" . $formvars['interval'][1] . "'";
            }
        }

        $sql.=" ORDER BY bon_achat_date_limit";

        if (empty($champs)) {
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }

    /**
     * Permet de savoir si un BA est valide pour etre utilisé
     *
     * @return Error
     *
     * @throws DediException
     */
    public function isValid()
    {
        if (empty($this->bon_achat_id)) {
            throw new DediException('You must load a BA before test if is valid');
        }

        $error = new Error();
        $texte = new Texte();

        // check validité
        if ($this->bon_achat_statut_id < self::BA_UTILISE) {
            $error->addTexte($texte->getOneOrCreate('err_ba_use', "Le bon d'achat a déjà été utilisé"));
        }
        // check date limite
        if ($this->bon_achat_date_limit < date('Y-m-d H:i:s')) {
            $error->addTexte($texte->getOneOrCreate('err_ba_date_limit', "La date limite d'utilisation du bon d'achat est passé"));
        }

        // check montant min
        $panier = Panier::getInstance();
        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();
        if ($aPayerTTC <= 0 || $aPayerTTC < $this->bon_achat_prix_min) {
            $error->addTexte(sprintf(
                $texte->getOneOrCreate('err_ba_montant_min', "Le montant minimum d'utilisation du bon d'achat est de %s EUR"),
                $this->bon_achat_prix_min
            ));
        }

        // check si le BA n'est pas deja dans le panier
        if ($panier->findBonAchat($this) !== false) {
            $error->addTexte($texte->getOneOrCreate('err_ba_in_cart', "La bon d'achat est déjà utilisé dans votre panier"));
        }

        return $error;
    }

    /**
     * Credite un BA sur le compte d'un client
     *
     * @param $formvars
     *
     * @throws DediException
     * @return int
     */
    function addBA($formvars) {

        if (empty($formvars['client_id'])) {
            throw new DediException('no client_id for add the BA');
        }

        if (!isset($formvars['nb_jour']))
            $formvars['nb_jour'] = self::BA_JOUR_DEFAULT;

        if (!isset($formvars['bon_achat_prix_min']))
            $formvars['bon_achat_prix_min'] = 0;

        $sql = "INSERT INTO bon_achat SET";
        $sql.=" client_id = '" . $formvars['client_id'] . "'";
        $sql.=", bon_achat_nom = '" . $this->sql->db->real_escape_string($formvars['bon_achat_nom']) . "'";
        $sql.=", bon_achat_prix = '" . $formvars['bon_achat_prix'] . "'";
        $sql.=", bon_achat_prix_min = '" . $formvars['bon_achat_prix_min'] . "'";
        $sql.=", bon_achat_date_ajout = NOW()";
        $sql.=", bon_achat_date_limit = DATE_ADD(NOW(), INTERVAL " . $formvars['nb_jour'] . " DAY)";
        $sql.=", bon_achat_statut_id = '" . self::BA_VALIDE . "'";

        $this->sql->query($sql);

        return $this->sql->db->insert_id;
    }

    /**
     * Fonction pour utiliser un BA pour payer une commande : passe un BA au statut utilisé
     *
     * @return int
     *
     * @throws DediException
     * @throws Exception
     */
    function useToPay() {

        // avec la serialization de l'objet, on perd la connection mysql => check si tjs connecté, si non on recup la connection
        /*if (!$this->sql->db->ping()) {
            $this->sql = SiteSQL::getInstance();
        }*/
        // TODO : tester si ca fonctionne
        $this->connection(SiteSQL::getInstance());

        if (empty($this->bon_achat_id)) {
            throw new DediException('You must load a BA before test if is valid');
        }

        $query = new Query();
        $query->update('bon_achat')->value('bon_achat_statut_id', self::BA_UTILISE)
            ->where('bon_achat_id', $this->bon_achat_id);

        $this->sql->query($query->build());
        return $this->sql->db->affected_rows;
    }
}
