<?php

/**
 * Class Boutique
 */
class Boutique
{

    /**
     * @var Boutique    An instance of Boutique
     */
    static protected  $instance;
    /**
     * @var int     Id of the boutique
     */
    public $id;
    /**
     * @var string  Name of the boutique
     */
    public $nom;
    /**
     * @var string  domain of the boutique
     */
    public $domaine;
    /**
     * @var string
     */
    public $dossier;
    /**
     * @var string
     */
    public $adminEmail;
    /**
     * @var string
     */
    public $contactEmail;
    /**
     * @var bool
     */
    public $tva;
    /**
     * @var array
     */
    public $footerPdf;
    /**
     * @var SiteSQL    An instance of Site_SQL, to get the connexion to DB
     */
    protected $sql;
    /**
     * @var Langue
     */
    protected $langue;

    /**
     * @var Devise
     */
    protected $devise;

    /**
     * @var array
     */
    protected $config;

    /**
     * Constructor
     */
    private function __construct()
    {
        $this->sql = SiteSQL::getInstance();
        $this->config = array();
    }

    /**
     * Check boutique
     *
     * @param int $boutiqueId id of the boutique
     *
     * @return bool
     */
    static public function isBoutique($boutiqueId)
    {
        return (Boutique::getInstance()->id == $boutiqueId);
    }

    /**
     * Get instance of Boutique
     *
     * @return null|Boutique
     */
    public static function getInstance()
    {
        if (!(self::$instance instanceof self)) {
            self::$instance = self::loadByHost($_SERVER['HTTP_HOST']);
        }
        return self::$instance;
    }

    static public function loadByHost($host)
    {
        $boutique = new self();
        $arrBoutique = $boutique->get(
            array('boutique_domaine' => $host)
        );

        if (empty($arrBoutique)) {
            throw new Exception('no boutique');
        }

        $boutique->init($arrBoutique);

        self::$instance = $boutique;
        return $boutique;
    }

    static public function loadById($boutiqueId)
    {
        $boutique = new self();
        $arrBoutique = $boutique->get(
            array('boutique_id' => $boutiqueId)
        );

        if (empty($arrBoutique)) {
            throw new Exception('no boutique');
        }

        $boutique->init($arrBoutique);

        self::$instance = $boutique;
        return $boutique;
    }

    /**
     * Init boutique
     *
     * @throws Exception
     * @return void
     */

    protected function init($arrBoutique)
    {
        $this->id = $arrBoutique[0]['boutique_id'];
        $this->nom = $arrBoutique[0]['boutique_nom'];
        $this->domaine = $arrBoutique[0]['boutique_domaine'];
        $this->dossier = $arrBoutique[0]['boutique_dossier'];
        $this->adminEmail = $arrBoutique[0]['boutique_admin_email'];
        $this->contactEmail = $arrBoutique[0]['boutique_contact_email'];
        $this->tva = $arrBoutique[0]['boutique_tva'];
        $this->footerPdf = $arrBoutique[0]['boutique_footer_pdf'];

        // langue
        $this->langue = new Langue($arrBoutique[0]['langue_id']);

        // devise
        $this->devise = new Devise($arrBoutique[0]['devise_id']);

        // load config
        $this->config = $this->loadConfig($this->id);
    }

    /**
     * init the statut of all data by boutique when create a new Boutique
     *
     * @param int   $boutiqueId id of the boutique
     * @param array $tables     list of tables name
     *
     * @return void
     */

    static public function initDB($boutiqueId, $tables = array())
    {
        $con = SiteSQL::getInstance();
        if (empty($tables)) {
            $con->query("SHOW TABLES", SiteSQL::SQL_ALL);
            $tables = $con->record;
        }

        $suffixe = '_boutique';
        $query = new Query();

        foreach ($tables as $t) {
            if (strpos($t[0], $suffixe) !== false) {
                $field = str_replace($suffixe, '', $t[0]);
                $query->reset()->delete($t[0])->where('boutique_id', $boutiqueId);
                $con->query($query->build());

                // insert lines for each records
                $sql ="INSERT INTO " . $t[0];
                $sql.=" (boutique_id, ".$field."_id, ".$field."_boutique_statut_id)";
                $sql.=" SELECT '".$boutiqueId."', ".$field."_id, 1 FROM " . $field;
                $con->query($sql);
            }
        }
    }

    static public function delete($boutiqueId, $tables = array())
    {
        $con = SiteSQL::getInstance();
        if (empty($tables)) {
            $con->query("SHOW TABLES", SiteSQL::SQL_ALL);
            $tables = $con->record;
        }

        $suffixe = '_boutique';
        $query = new Query();

        foreach ($tables as $t) {
            if (strpos($t[0], $suffixe) !== false) {
                $query->reset()->delete($t[0])->where('boutique_id', $boutiqueId);
                $con->query($query->build());
            }
        }

        $query->reset()->delete('boutique')->where('boutique_id', $boutiqueId);
        $con->query($query->build());
    }

    /**
     * Get Devise of the boutique
     *
     * @return Devise
     */
    public function getDevise()
    {
        return $this->devise;
    }

    /**
     * Setter for devise
     *
     * @param Devise $devise the devise object
     *
     * @return void
     */
    public function setDevise(Devise $devise)
    {
        $this->devise = $devise;
    }

    /**
     * Getter for Langue
     *
     * @return Langue
     */
    public function getLangue()
    {
        return $this->langue;
    }

    /**
     * Setter for Langue
     *
     * @param Langue $langue the Langue object
     *
     * @return void
     */
    public function setLangue(Langue $langue)
    {
        $this->langue = $langue;
    }

    /**
     * Create entry statut in bdd for the table pass in parameter
     *
     * @param string $table   name of the table
     * @param int    $tableId id of the record for create the statut
     *
     * @return void
     */
    public function createStatut($table, $tableId)
    {
        $boutiques = $this->get();

        foreach ($boutiques as $b) {
            $sql = "INSERT INTO ".$table."_boutique SET";
            $sql.= " ".$table."_id = '".$tableId."'";
            $sql.= ", boutique_id = '".$b['boutique_id']."'";
            $sql.= ", ".$table."_boutique_statut_id = '1'";
            $this->sql->query($sql);
        }
    }

    /**
     * Get Boutique or list of Boutique
     *
     * @param array  $formvars list of param to get boutique
     * @param string $champs   name of the field to get the value
     *
     * @return array|string
     */
    public function get($formvars = array(), $champs = "")
    {
        $sql="SELECT *";
        $sql.=" FROM boutique b";
        $sql.=" LEFT JOIN langue l ON(b.langue_id = l.langue_id)";
        $sql.=" LEFT JOIN devise d ON(b.devise_id = d.devise_id)";
        $sql.=" WHERE 1";

        if (!empty($formvars['boutique_id'])) {
            $sql .= " AND b.boutique_id = '" . $formvars['boutique_id'] . "'";
        }
        if (!empty($formvars['boutique_domaine'])) {
            $sql .= " AND b.boutique_domaine = '" . $formvars['boutique_domaine'] . "'";
        }
        if (!empty($formvars['boutique_email'])) {
            $sql .= " AND b.boutique_email = '" . $formvars['boutique_email'] . "'";
        }
        if (!empty($formvars['boutique_password'])) {
            $sql .= " AND b.boutique_password = '" . $formvars['boutique_password'] . "'";
        }

        $sql.=" ORDER BY b.boutique_id";


        if (empty($champs)) {
            $this->sql->query($sql, SiteSQL::SQL_ALL, SiteSQL::SQL_ASSOC);
            return $this->sql->record;
        } else {
            $this->sql->query($sql, SiteSQL::SQL_INIT, SiteSQL::SQL_ASSOC);
            return $this->sql->record[$champs];
        }
    }

    /**
     * @param string $key
     *
     * @return array
     * @throws DediException
     */
    public function getConfig($key = '', $throwException = true)
    {
        if (!empty($key)) {
            if (array_key_exists($key, $this->config) == false && $throwException) {
                throw new DediException('no key ' . $key . ' in config');
            }
            if(isset($this->config[$key])){
                return $this->config[$key];
            }
        }

        return $this->config;
    }

    /**
     * @param $boutiqueId
     *
     * @return array
     * @throws Exception
     */
    public function loadConfig($boutiqueId)
    {
        $query = new Query();
        $query->select()->from('boutique_config')->where('boutique_id', $boutiqueId);

        $results = $this->sql->getAll($query->build());

        $data = array();

        foreach ($results as $result) {
            $path = explode('.', $result['boutique_config_key']);

            $arr = array();
            $tmp = &$arr;
            foreach ($path as $segment) {
                $tmp[$segment] = array();
                $tmp = &$tmp[$segment];
            }
            $tmp = $result['boutique_config_value'];
            $data = array_merge_recursive($data, $arr);
        }

        return $data;
    }

    /**
     * return the name of the route
     *
     * @return string
     */
    public function getNextPageProcess()
    {
        // TODO : modif pour le mobile
        $deviceType = (Application::getMobileDetect()->isMobile() && !Application::getMobileDetect()->isTablet()) ?
            'mobile' : 'desktop';

        return ($this->config['process']['etape'] == 'simple' && $deviceType == 'desktop') ? "order" : "coordonnees";
    }

    public function __destruct()
    {
        // TODO : revoir => exemple sur page panier
        $_SESSION['langue_id'] = $this->getLangue()->id;
        $_SESSION['langue_ini'] = $this->getLangue()->ini;
        $_SESSION['devise_id'] = $this->getDevise()->id;
    }

}