<?php

/**
 * Class Caracteristique
 */
class Caracteristique extends Model {

    const NOTE_PARKER = 1;
    const NOTE_WS = 6;
    const NOTE_JMQ = 7;
    const NOTE_MILL = 8;
    const APPELLATION = 9;

    protected $table = "caracteristique";

    // recupere les infos d'une cara
    function get($formvars = array(), $champs = '') {
        $sql = "SELECT * FROM caracteristique c, caracteristique_nom cn";
        $sql.=" WHERE c.caracteristique_id = cn.caracteristique_id";
        $sql.=" AND cn.langue_id = '" . Boutique::getInstance()->getLangue()->id . "'";

        if (empty($formvars['all_cara']))
            $sql.=" AND c.caracteristique_statut_id = '1'";
        if (!empty($formvars['caracteristique_id']))
            $sql.=" AND c.caracteristique_id = '" . $formvars['caracteristique_id'] . "'";

        $sql .= " ORDER BY c.caracteristique_rang, cn.caracteristique_nom_nom";

        if (empty($champs)) {
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }

    function getCaraProduit($formvars = array(), $champs = '') {

        $lgId = Boutique::getInstance()->getLangue()->id;

        $sql = "SELECT *, GROUP_CONCAT(cvn.caracteristique_valeur_nom_nom SEPARATOR ', ') as nom_cara FROM (caracteristique c, caracteristique_nom cn, caracteristique_valeur cv, caracteristique_valeur_nom cvn)";
        $sql.=" LEFT JOIN produit_caracteristique_valeur pcv ON pcv.caracteristique_valeur_id = cv.caracteristique_valeur_id";
        $sql.=" WHERE c.caracteristique_id = cn.caracteristique_id";
        $sql.=" AND cv.caracteristique_id = c.caracteristique_id";
        $sql.=" AND cvn.caracteristique_valeur_id = cv.caracteristique_valeur_id";
        $sql.=" AND cn.langue_id = '" . $lgId . "'";
        $sql.=" AND cvn.langue_id = '" . $lgId . "'";

        if (!empty($formvars['produit_id']))
            $sql.=" AND pcv.produit_id = '" . $formvars['produit_id'] . "'";
        $sql .= " GROUP BY c.caracteristique_id";
        $sql .= " ORDER BY c.caracteristique_rang, cn.caracteristique_nom_nom";

        if (empty($champs)) {
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }

    /**
     * @param array $formvars
     * @param string $champs
     * @return null
     */
    function getValeur($formvars = array(), $champs = '') {

        $lgId = Boutique::getInstance()->getLangue()->id;

        $sql = "SELECT * ";
        $sql.=" FROM (caracteristique_valeur cv, caracteristique_valeur_nom cvn, caracteristique c)";
        $sql.=" LEFT JOIN caracteristique_nom cn ON (c.caracteristique_id = cn.caracteristique_id AND cn.langue_id = '" . $lgId . "')";

        $sql.=" WHERE cv.caracteristique_valeur_id = cvn.caracteristique_valeur_id";
        $sql.=" AND cvn.langue_id = '" . $lgId . "'";
        $sql.=" AND c.caracteristique_id = cv.caracteristique_id";

        if (!empty($formvars['filtre']))
            $sql.=" AND cv.caracteristique_valeur_visible_filtre = '1'";
        if (empty($formvars['all_cara']))
            $sql.=" AND c.caracteristique_statut_id = '1'";
        if (!empty($formvars['caracteristique_valeur_id']))
            $sql.=" AND cv.caracteristique_valeur_id = '" . $formvars['caracteristique_valeur_id'] . "'";
        if (!empty($formvars['list_caracteristique_valeur_id']))
            $sql.=" AND cv.caracteristique_valeur_id IN (" . $formvars['list_caracteristique_valeur_id'] . ")";
        if (!empty($formvars['caracteristique_id']))
            $sql.=" AND cv.caracteristique_id = '" . $formvars['caracteristique_id'] . "'";
        if (!empty($formvars['search']))
            $sql.=" AND cvn.caracteristique_valeur_nom_nom LIKE '%" . $formvars['search'] . "%'";

        //$sql.=" GROUP BY cv.caracteristique_valeur_id";
        $sql.=" ORDER BY cv.caracteristique_valeur_rang";

        if (empty($champs)) {
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }

    // Recupere les valeurs d'une liste de produit
    function getValeurProduit($formvars = array(), $champs = '') {

        if (empty($formvars['listProduit'])) {
            return false;
        }

        $lgId = Boutique::getInstance()->getLangue()->id;

        $sql = "SELECT c.caracteristique_id, c.caracteristique_ouverture, cn.caracteristique_nom_nom, cv.caracteristique_valeur_id, cvn.caracteristique_valeur_nom_nom, cv.caracteristique_valeur_rang, c.caracteristique_rang, cvn.caracteristique_valeur_nom_image";
        $sql.=", pcv.produit_id";
        $sql.=" FROM (produit_caracteristique_valeur pcv)";
        $sql.=" LEFT JOIN caracteristique_valeur cv ON (pcv.caracteristique_valeur_id = cv.caracteristique_valeur_id)";
        $sql.=" LEFT JOIN caracteristique_valeur_nom cvn ON (cv.caracteristique_valeur_id = cvn.caracteristique_valeur_id AND cvn.langue_id = '" . $lgId . "')";
        $sql.=" LEFT JOIN caracteristique c ON (cv.caracteristique_id = c.caracteristique_id)";
        $sql.=" LEFT JOIN caracteristique_nom cn ON (c.caracteristique_id = cn.caracteristique_id AND cn.langue_id = '" . $lgId . "')";

        $sql.=" WHERE 1";

        if (empty($formvars['all_cara'])) {
            $sql .= " AND c.caracteristique_statut_id = '1'";
        }

        $sql.=" AND pcv.produit_id IN (" . implode(', ', $formvars['listProduit']) . ")";

        if (!empty($formvars['caracteristique_id'])) {
            if (is_array($formvars['caracteristique_id'])) {
                $sql .= " AND cv.caracteristique_id IN (" . implode(',', $formvars['caracteristique_id']) . ")";
            } else {
                $sql .= " AND cv.caracteristique_id = '" . $formvars['caracteristique_id'] . "'";
            }
        }

        if (empty($formvars['group']))
            $formvars['group'] = '';
        switch ($formvars['group']) {
            case "pcv":
                $sql .= " GROUP BY pcv.produit_caracteristique_valeur_id";
                break;
            default:
                $sql .= " GROUP BY pcv.caracteristique_valeur_id";
                break;
        }

        $sql .= " ORDER BY c.caracteristique_rang, cn.caracteristique_nom_nom, cv.caracteristique_valeur_rang, cvn.caracteristique_valeur_nom_nom";

        if (empty($champs)) {
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }

    function parseUrl($url) {

        $arrUrlCara = explode(',', $url);
        $listCara = array();
        foreach ($arrUrlCara as $cara) {
            if (preg_match('/([a-zA-Z0-9-_]+)-([0-9]+)/', $cara, $matches)) {
                array_push($listCara, $matches[2]);
            }
        }
        $listCara = array_unique($listCara);
        return $listCara;
    }

    function parseSql($listCara) {

        // on associe les cara entre elle
        $listCaraSort = array();
        foreach ($listCara as $cara) {
            $sql = "SELECT caracteristique_id FROM caracteristique_valeur WHERE caracteristique_valeur_id = '" . $cara . "'";
            $result = $this->sql->getOne($sql);
            $idCara = $result['caracteristique_id'];
            if (!isset($listCaraSort[$idCara]))
                $listCaraSort[$idCara] = array();
            array_push($listCaraSort[$idCara], $cara);
        }

        return $listCaraSort;
    }

    /**
     * @param array $cara
     * @param CaracteristiqueValeur[] $caraUrl
     * @param string $url
     */
    function setUrl(&$cara, $caraUrl, $url) {

        if(strpos($url, '?') !== false) {
            list($debutUrl, $finUrl) = explode('?', $url);
        } else {
            $debutUrl = $url;
            $finUrl = '';
        }

        foreach ($cara as &$c) {
            $newUrlCara = "";
            if (!empty($caraUrl)) {
                $add = false;
                foreach ($caraUrl as $val) {
                    if ($val->caracteristique_valeur_id == $c['caracteristique_valeur_id']) {
                        $add = true;
                        continue;
                    }
                    if ($val->caracteristique_rang >= $c['caracteristique_rang'] && (($val->caracteristique_id == $c['caracteristique_id'] && $val->caracteristique_valeur_rang > $c['caracteristique_valeur_rang']) || $val->caracteristique_id != $c['caracteristique_id']) && $add == false) {
                        $newUrlCara.="," . Tools::parseUrl($c['caracteristique_nom_nom']) . "-" . Tools::parseUrl($c['caracteristique_valeur_nom_nom']) . "-" . $c['caracteristique_valeur_id'];
                        $add = true;
                    }
                    $newUrlCara.="," . Tools::parseUrl($val->caracteristique_nom_nom) . "-" . Tools::parseUrl($val->caracteristique_valeur_nom_nom) . "-" . $val->caracteristique_valeur_id;
                }
                if ($add == false) {
                    $newUrlCara.="," . Tools::parseUrl($c['caracteristique_nom_nom']) . "-" . Tools::parseUrl($c['caracteristique_valeur_nom_nom']) . "-" . $c['caracteristique_valeur_id'];
                }
            } else {
                $newUrlCara.="," . Tools::parseUrl($c['caracteristique_nom_nom']) . "-" . Tools::parseUrl($c['caracteristique_valeur_nom_nom']) . "-" . $c['caracteristique_valeur_id'];
            }

            if(!empty($newUrlCara)) {
                $newUrlCara = '/' . substr($newUrlCara, 1, strlen($newUrlCara));
            }

            $c['url'] = $debutUrl . $newUrlCara;

            $finUrl = preg_replace('/&page=([0-9]+)/', '', $finUrl);
            if (!empty($finUrl))
                $c['url'].='?' . $finUrl;
        }
    }

    function constructCara($arrValeur, $arrCaraAff = array(), $arrProd = array()) {

        $arrCaracteristique = $this->get();

        // recup cara a laissé active
        if (!empty($arrProd)) {
            $listID = array();
            foreach ($arrProd as $p) {
                $listID[] = $p['produit_id'];
            }
            $arrCaracActive = $this->getValeurProduit(array('listProduit' => $listID));
            $listCaraActive = array();
            foreach ($arrCaracActive as $carActive) {
                $listCaraActive[] = $carActive['caracteristique_valeur_id'];
            }
        }
        //--
        if(empty($arrValeur))
            $arrValeur = array();

        foreach ($arrCaracteristique as $key => $value) {
            foreach ($arrValeur as $v) {
                if ($arrCaracteristique[$key]['caracteristique_id'] == $v['caracteristique_id']) {
                    // Test si l'option a ete cochee
                    if (!empty($arrCaraAff)) {
                        foreach ($arrCaraAff as $aff) {
                            if ($aff->caracteristique_valeur_id == $v['caracteristique_valeur_id']) {
                                $v['exist'] = 1;
                                $arrCaracteristique[$key]['display'] = 1;
                            }
                        }
                    }
                    // --
                    // test si il faut grise ou non l'option pr eviter d'avoir des listings vide
                    if (!empty($listCaraActive)) {
                        if (!in_array($v['caracteristique_valeur_id'], $listCaraActive)) {
                            $v['url'] = '';
                        }
                    }
                    //--

                    $arrCaracteristique[$key]['valeur'][] = $v;
                }
            }
        }

        return $arrCaracteristique;
    }

    function add($formvars = array()) {

        Tools::escapeDataToSql($formvars);

        // on regarde si la cara existe ou pas
        $find_opt = "SELECT cv.caracteristique_valeur_id";
        $find_opt .=" FROM caracteristique_valeur_nom cvn, caracteristique_valeur cv";
        $find_opt .=" WHERE cv.caracteristique_valeur_id = cvn.caracteristique_valeur_id";
        $find_opt .="  AND cv.caracteristique_id = '" . $formvars['caracteristique_id'] . "'";
        $find_opt .="  AND cvn.caracteristique_valeur_nom_nom = '" . $formvars['caracteristique_valeur_nom_nom'] . "'";
        $find_opt .="  AND cvn.langue_id = '" . $formvars['langue_id'] . "'";
        $arrCara = $this->sql->getOne($find_opt);
        if (empty($arrCara)) {

            $sql = "INSERT INTO caracteristique_valeur SET";
            $sql.=" caracteristique_id = '" . $formvars['caracteristique_id'] . "'";
            $sql.=", caracteristique_valeur_statut_id = 1";
            $sql.=", caracteristique_valeur_visible_filtre = '" . $formvars['caracteristique_valeur_visible_filtre'] . "'";
            $sql.=", caracteristique_valeur_rang = '" . $formvars['caracteristique_valeur_rang'] . "'";
            $this->sql->query($sql);
            $idValeur = $this->sql->db->insert_id;

            $sql = "INSERT INTO caracteristique_valeur_nom SET";
            $sql.=" caracteristique_valeur_id = '" . $idValeur . "'";
            $sql.=", caracteristique_valeur_nom_nom = '" . $formvars['caracteristique_valeur_nom_nom'] . "'";
            $sql.=", langue_id = '" . $formvars['langue_id'] . "'";
            $this->sql->query($sql);
        } else {
            $idValeur = $arrCara['caracteristique_valeur_id'];
        }


        $find_cara = "SELECT produit_caracteristique_valeur_id";
        $find_cara .= " FROM produit_caracteristique_valeur";
        $find_cara .= " WHERE produit_id = '" . $formvars['produit_id'] . "'";
        $find_cara .= " AND caracteristique_valeur_id = '" . $idValeur . "'";
        $TheCara = $this->sql->getOne($find_cara);

        if (empty($TheCara) && !empty($formvars['produit_id'])) {
            $sql_create = "INSERT INTO produit_caracteristique_valeur SET";
            $sql_create.= " produit_id = '" . $formvars['produit_id'] . "'";
            $sql_create.=", caracteristique_valeur_id = '" . $idValeur . "'";
            $this->sql->query($sql_create);
        }

        return $idValeur;
    }

    function setCanonical($arrCaraAff, $url) {

        $caraCanonical = array();
        $caraTraite = array();
        // on construit le tableau simple
        foreach ($arrCaraAff as $cara) {
            if (!in_array($cara['caracteristique_id'], $caraTraite)) {
                array_push($caraTraite, $cara['caracteristique_id']);
                array_push($caraCanonical, $cara['caracteristique_valeur_id']);
            }
        }

        $newUrl = $url;

        $arrUrlCara = explode(',', $url);
        $listCara = array();
        foreach ($arrUrlCara as $cara) {
            if (preg_match('/([a-zA-Z0-9-_]+)-([0-9]+)/', $cara, $matches)) {
                if (!in_array($matches[2], $caraCanonical)) {
                    array_push($listCara, $matches[2]);
                    $newUrl = str_replace(',' . $matches[0], '', $newUrl);
                }
            }
        }

        if ($newUrl != $url)
            return $newUrl;
        else
            return '';
    }

    /**
     *
     * php dedi execute:method Caracteristique updateCatalog
     *
     * @throws DediException
     * @throws Exception
     */
    public function updateCatalog()
    {
        $query = new Query();
        $query->select()->from('produit');
        $results = $this->sql->getAll($query->build());

        $produit = new Produit();
        foreach ($results as $result) {
            $produit->updateCaracteristique($result['produit_id']);
        }
    }

}