<?php

use Tree\Tree;
use Tree\Node;

use Symfony\Component\Routing\Exception\InvalidParameterException;

/**
 * Class Categorie
 *
 * @property integer $categorie_id
 * @property integer $parent_id
 * @property integer $categorie_rang
 * @property integer $categorie_visible
 * @property string $categorie_nom_nom
 * @property string $categorie_nom_desc
 */
class Categorie extends Model {

//    const BORDEAUX = 1;

    protected $table = 'categorie';

    static protected $tree = null;

    function get($formvars = array(), $champs = "") {

        $lgId = Boutique::getInstance()->getLangue()->id;

        $sql = "SELECT *, c.parent_id, c.categorie_id, c.categorie_image, cnpar.categorie_nom_nom as parent_nom_nom";
        if (!empty($formvars['products']) || !empty($formvars['nbprod'])){
            $sql.=", COUNT(pc.produit_id) as nbProd";
        }

        $sql.=" FROM (categorie c)";
        if(empty($formvars['no_boutique'])) {
            $sql.=" INNER JOIN categorie_boutique cb ON (c.categorie_id = cb.categorie_id AND cb.boutique_id = '".Boutique::getInstance()->id."')";
        }
        $sql.=" LEFT JOIN categorie_nom cnpar ON (cnpar.categorie_id = c.parent_id AND cnpar.langue_id='" . $lgId . "')";
        $sql.=" LEFT JOIN categorie_nom cn ON (c.categorie_id=cn.categorie_id AND cn.langue_id='" . $lgId . "')";
        if (!empty($formvars['products']) || !empty($formvars['nbprod'])){
            $sql.=" LEFT JOIN produit_categorie pc ON (pc.categorie_id = c.categorie_id)";
        }

        $sql.= " WHERE c.categorie_visible = '1'";
        if(empty($formvars['no_boutique'])) {
            $sql.= " AND cb.categorie_boutique_statut_id = '1'";
        }
        if (!empty($formvars['categorie_boutique_home_visible']))
            $sql.=" AND cb.categorie_boutique_home_visible = '" . $formvars['categorie_boutique_home_visible'] . "'";
        if (!empty($formvars['categorie_id']))
            $sql.=" AND c.categorie_id = '" . $formvars['categorie_id'] . "'";
        if (!empty($formvars['categorie_id_not_in']))
            $sql.=" AND c.categorie_id NOT IN (" . $formvars['categorie_id_not_in'] . ")";
        if (!empty($formvars['parent_id']))
            $sql.=" AND c.parent_id = '" . $formvars['parent_id'] . "'";
        if (!empty($formvars['only_parent']))
            $sql.=" AND c.parent_id IS NULL";
        if (!empty($formvars['no_parent']))
            $sql.=" AND c.parent_id IS NOT NULL";

        $sql.=" GROUP BY c.categorie_id";

        if (!empty($formvars['products'])){
            $sql.=" HAVING nbProd > 0";
        }

        if (empty($formvars['order']))
            $formvars['order'] = '';

        switch ($formvars['order']) {
            case 'nom':
                $sql.=" ORDER BY cn.categorie_nom_nom";
                break;
            case 'nbprod':
                $sql.=" ORDER BY nbProd ASC";
                break;
            case 'rang_desc':
                $sql.=" ORDER BY c.categorie_rang DESC";
                break;
            default:
                $sql.=" ORDER BY c.categorie_rang";
        }

        if (empty($champs)) {
            $this->sql->query($sql, SiteSQL::SQL_ALL, SiteSQL::SQL_ASSOC);
            return $this->sql->record;
        } else {
            $this->sql->query($sql, SiteSQL::SQL_INIT, SiteSQL::SQL_ASSOC);
            return $this->sql->record[$champs];
        }
    }

    // mise a jour des identifants etendus par categorie
    function updateExtId($categorie_id) {

        $categorie_ext_id = $categorie_id;
        $arrCategorie[0]['parent_id'] = $categorie_id;
        do {

            $arrCategorie = $this->get(array('categorie_id' => $arrCategorie[0]['parent_id']));
            if (!empty($arrCategorie[0]['parent_id'])) {
                $categorie_ext_id.="," . $arrCategorie[0]['parent_id'];
            }
        } while (!empty($arrCategorie[0]['parent_id']));

        $sql = " UPDATE categorie SET categorie_ext_id = '" . $categorie_ext_id . "'";
        $sql.= " WHERE categorie_id = '" . $categorie_id . "'";
        $this->sql->query($sql);

        // on met a jour les enfants
        $sql = "SELECT categorie_id FROM categorie WHERE parent_id = '" . $categorie_id . "'";
        $this->sql->query($sql, SiteSQL::SQL_ALL, SiteSQL::SQL_ASSOC);
        $arrEnfant = $this->sql->record;
        if (!empty($arrEnfant)) {
            foreach ($arrEnfant as $enfant) {
                $this->updateExtId($enfant['categorie_id']);
            }
        }
    }

    /**
     * Retourne un arbre avec toutes les categories du site
     *
     * @param array $formvars
     * @return null|Tree
     */
    public static function getTree($formvars = array())
    {
        if (is_null(self::$tree)) {
            self::$tree = new Tree();

            $categorie = new Categorie();
            $categories = $categorie->get();

            foreach ($categories as $cat) {
                if (empty($cat['parent_id'])) {
                    self::$tree->addNode(new Node($cat));
                } else {
                    $parent = self::$tree->findNode(array('categorie_id' => $cat['parent_id']));
                    if ($parent instanceof Node) {
                        $parent->addChildren(new Node($cat));
                    }
                }
            }
        }
        if (!empty($formvars)) {
            return self::$tree->findNode($formvars);
        } else {
            return self::$tree;
        }
    }

    /**
     * @param $categorieId
     */
    public static function buildBreadcrumb($categorieId)
    {
        $bd = Breadcrumb::getInstance();

        do {
            $cat = new Categorie($categorieId);
            $treeCat[] = $cat;
            $categorieId = $cat->parent_id;
        } while (!empty($categorieId));

        $treeCat = array_reverse($treeCat);

        $router = Application::getRouter()->getRouter();

        foreach ($treeCat as $k => $tc) {

            $paramUrl = array();
            $paramUrl['categorie_nom'] = Tools::parseURL($tc->categorie_nom_nom);
            $paramUrl['id'] = $tc->categorie_id;
            if (!empty($treeCat[$k-1])) {
                $paramUrl['parent_nom'] = Tools::parseURL($treeCat[$k-1]->categorie_nom_nom);
            }
            if (!empty($treeCat[$k-2])) {
                $paramUrl['grand_parent_nom'] = Tools::parseURL($treeCat[$k-2]->categorie_nom_nom);
            }

            //if ($k+1 > 1) {
                try {
                    $url = $router->generate('liste_' . ($k + 1), $paramUrl);
                } catch (InvalidParameterException $e) {
                    $debugBar = Application::getDebugBar();
                    if (!empty($debugBar)) {
                        $debugBar['exceptions']->addException($e);
                    }
                    $url = '';
                }
            //} else {
            //    $url = '';
            //}
            $bd->add($tc->categorie_nom_nom, $url);
        }
    }
}
