<?php

namespace Console\Command;

use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Command\Command;

use RecursiveIteratorIterator;
use RecursiveDirectoryIterator;

class CacheClear extends Command
{
    protected function configure()
    {
        $this
            ->setName('cache:clear')
            ->setDescription('vide le cache / fichier compilé')
            ->addOption(
                'compiled',
                null,
                InputOption::VALUE_NONE,
                'Si défini, efface uniquement les templates compilés'
            )
            ->addOption(
                'cache',
                null,
                InputOption::VALUE_NONE,
                'Si défini, efface uniquement les fichiers de cache'
            )->addOption(
                'php',
                null,
                InputOption::VALUE_NONE,
                'Si défini, efface uniquement le cache PHP'
            );
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {

        $clearCompiled = $clearCache = $clearPhp = (!$input->getOption('compiled') && !$input->getOption('cache') && !$input->getOption('php'));

        if ($input->getOption('compiled')) {
            $clearCompiled = true;
        }

        if ($input->getOption('cache')) {
            $clearCache = true;
        }

        if ($input->getOption('php')) {
            $clearPhp = true;
        }

        if ($clearCompiled) {
            $this->clearDirectory(SITE_DIR . 'storage/smarty/compiled/');
            $output->writeln("<info>clear Smarty compiled</info>");
        }

        if ($clearCache) {
            $this->clearDirectory(SITE_DIR . 'storage/smarty/cache/');
            $output->writeln("<info>clear Smarty cache</info>");
        }

        if ($clearPhp) {
            $this->clearDirectory(SITE_DIR . 'storage/php/');
            $output->writeln("<info>clear Php cache</info>");
        }
    }

    protected function clearDirectory($dir, $mask = '*')
    {
        $iterator = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($dir), RecursiveIteratorIterator::SELF_FIRST);
        foreach($iterator as $item) {
            $itemName = basename($item);
            if ($itemName == '.' || $itemName == '..') {
                continue;
            }

            if (is_dir($item)) {
                array_map("unlink", glob($item . '/' . $mask));
            }
        }
    }
}