<?php

namespace Controller;

use Application;
use Auth\AuthException;
use Boutique;
use Cms;
use Exception;
use Form\AddressForm;
use Form\Constraint\NotBlank;
use Form\Field;
use Form\ParrainageForm;
use Form\PasswordForm;
use Form\ProfileForm;
use Mailer;
use Pays;
use Shopbydedi\Client;
use SimplePie;
use Texte;
use Breadcrumb;
use Adresse;
use BonAchat;
use Commande;
use Produit;
use Tools;
use Wishlist;
use Livraison;

use Symfony\Component\Routing\Annotation\Route;

/**
 * @Route("/account", defaults = {"role" = "customer"})
 */
class AccountController extends Controller
{

    /**
     * @Route("/", name="compte")
     *
     * @return mixed
     */
    public function account()
    {
        $texte = new Texte();
        Breadcrumb::getInstance()->add($texte->getOneOrCreate('mon_compte', 'Mon compte'));

        // recup articles blog
        $config = Boutique::getInstance()->getConfig();
        if (!empty($config['blog']['feed_url'])) {
            $feed = new SimplePie();
            $feed->set_feed_url($config['blog']['feed_url']);
            $feed->set_cache_location(SITE_DIR . 'storage/php/' . Boutique::getInstance()->dossier);
            $feed->init();
            $articles = $feed->get_items(0, $config['blog']['limit']);
            $this->template->assign('articles', $articles);
        }

        return $this->render('account/index.tpl');
    }

    /**
     * @Route("/modify", name="modif_compte", methods = { "GET" })
     *
     * @return mixed
     */
    public function modify()
    {
        $texte = new Texte();
        $bd = Breadcrumb::getInstance();
        $bd->add($texte->getOneOrCreate('mon_compte', 'Mon compte'));
        $bd->add($texte->getOneOrCreate('mon_profil', 'Mon profil'));

        $pays = new Pays();
        $client = $this->getCurrentClient();

        $tplVars = array();
        $tplVars['pays'] = $pays->get();
        $tplVars['civilite'] = $client->getCivilite();
        $tplVars['client'] = $client->toArray();

        return $this->render('account/modify.tpl', array_merge($this->getFlashAssign(array("error", "success", "post")),$tplVars));
    }

    /**
     * @Route("/modify", name="modif_compte_post", methods = { "POST" })
     *
     */
    public function modify_post()
    {
        $texte = new Texte();
        $post = $this->request->request->all();
        $session = $this->request->getSession();
        $flashBag = $session->getFlashBag();
        $form = new ProfileForm();
        // init de la date
        if(!empty($post['client_ddn_Year']) && !empty($post['client_ddn_Month']) && !empty($post['client_ddn_Day'])) {
            $date = $post['client_ddn_Year'] . "-" . str_pad($post['client_ddn_Month'], 2, "0", STR_PAD_LEFT) . "-" . str_pad($post['client_ddn_Day'], 2, "0", STR_PAD_LEFT);
            $this->request->request->set('client_ddn',$date);
            $post['client_ddn'] = $date;
        }

        if ($error = $form->handle($this->request)) {
            $flashBag->add("error", $error);
        }

        if (!$flashBag->has('error')) {
            // get the current client
            $client = $this->getCurrentClient();
            $post['client_id'] = $this->getCurrentClient()->client_id;
            // if parrain email, get its id
            if(!empty($post["client_referer"])) {
                $parrainId = $client->get(array("client_email" => $post["client_referer"]), "client_id");
                if(empty($parrainId)) {
                    $flashBag->add('error',
                        $texte->getOneOrCreate('errCreationClient',
                            'Une erreur lors la récupération du parrain est survenue. Veuillez re essayer.')
                    );
                } else {
                    $post["parrain_id"] = $parrainId;
                }
            }
            try {
                $client->update($post);
            } catch (\Exception $e) {
                $flashBag->add('error', $texte->getOneOrCreate('nokModifCompte',
                    'Une erreur a empêché la mise à jour de votre compte.'));
            }
            $flashBag->add('success', $texte->getOneOrCreate('okModifCompte', 'Votre compte a été mis à jour'));
        } else {
            // save param in flashbag
            $flashBag->set('post', $post);
        }

        return $this->redirect($this->generateUrl('modif_compte'));
    }

    /**
     * @Route("/account/rollover", name="account_rollover")
     */
    public function rollover()
    {
        //@TODO: appeler la fonction qui sert à savoir si un client est connecté ou non
        $session = $this->request->getSession();
        if($session->get('client_id')){
            $connect = true;
            $customer = new Client((int)$_SESSION['client_id']);
            $this->template->assign(array(
                'customer' => $customer
            ));

        } else {
            $connect = false;
        }
        $this->template->assign('connect', $connect);

        return $this->response(
            $this->template->fetch('account/rollover.tpl')
        );
    }

    /**
     * @Route("/wishlist", name="compte_wishlist")
     */
    public function wishlist()
    {
        $texte = new Texte();
        $bd = Breadcrumb::getInstance();
        $bd->add($texte->getOneOrCreate('mon_compte', 'Mon compte'));
        $bd->add($texte->getOneOrCreate('ma_wishlist', 'Ma_Wishlist'));

        $session = $this->request->getSession();
        $produit = new Produit();
        $wishlist = new Wishlist($session->get('client_id'));
        $produits = $wishlist->getProduits();

        $listProd = array();
        if (!empty($produits)) {
            foreach ($produits as $prod) {
                // recup produit
                list($pTemp) = $produit->getBloc(array('produit_id' => $prod['produit_id'], 'produit_attribut_id' => $prod['produit_attribut_id']));
                $pTemp['produit_list_produit_id'] = $prod['produit_list_produit_id'];

                // recup couleur
                $pTemp['couleurs'] = $produit->getOption(array(
                    "produit_option_id" => Produit::$idCouleur,
                    "produit_id" => $prod['produit_id'],
                    "group" => "couleur"
                ));

                $listProd[] = $pTemp;
            }
        }

        $this->template->assign('produits', $listProd);

        return $this->render('account/wishlist.tpl');
    }

    /**
     * @Route("/ba", name="compte_ba")
     */
    public function ba()
    {
        $texte = new Texte;
        $bd = Breadcrumb::getInstance();
        $bd->add($texte->getOneOrCreate('mon_compte', 'Mon compte'));
        $bd->add($texte->getOneOrCreate('mes_ba', 'Mes bons d\'achats'));

        $bonAchat = new BonAchat;
        $arrBon = $bonAchat->get(array(
            'client_id' => $this->getCurrentClient()->client_id,
//            'bon_achat_statut_id' => BonAchat::BA_VALIDE -- commented this line so all the bon's are shown even if they are active or used
        ));
        $this->template->assign('bon', $arrBon);

        return $this->render('account/ba.tpl');
    }

    /**
     * @Route("/filleul", name="compte_filleul")
     */
    public function filleul()
    {
        $texte = new Texte();
        $bd = Breadcrumb::getInstance();
        $bd->add($texte->getOneOrCreate('mon_compte', 'Mon compte'));
        $bd->add($texte->getOneOrCreate('filleul', 'Filleuls'));

        $filleul = new Client;
        $arrFilleul = $filleul->get(array('parrain_id' => $this->getCurrentClient()->client_id));

        return $this->render('account/filleul.tpl', array("filleuls" =>  $arrFilleul));
    }

    /**
     * @Route("/parrainage", name="compte_parrainage", methods = { "GET" })
     */
    public function parrainage()
    {
        $texte = new Texte();
        $bd = Breadcrumb::getInstance();
        $bd->add($texte->getOneOrCreate('mon_compte', 'Mon compte'));
        $bd->add($texte->getOneOrCreate('parrainage', 'Parrainage'));
        return $this->render('account/parrainage.tpl', $this->getFlashAssign(array("error", "success", "post")));
    }

    /**
     * @Route("/parrainage", name="compte_parrainage_post", methods = { "POST" })
     */
    public function parrainage_post()
    {
        $form = new ParrainageForm();
        $flashBag = $this->getFlashBag();

        $client = $this->getCurrentClient();

        if($error = $form->handle($this->request)){
            $flashBag->add("error", $error);
        }

        if (!$flashBag->has('error')) {
            $mailer = new Mailer(Application::getEmailSender());
            $texte = new Texte();

            $indexes = array("femail1", "femail2", "femail3", "femail4", "femail5");
            $emails = array();
            foreach ($indexes as $idx) {
                if ($this->request->get($idx) != false && filter_var($this->request->get($idx), FILTER_VALIDATE_EMAIL) !== false) {
                    $emails[] = $this->request->request->get($idx);
                } elseif($this->request->get($idx) != false) {
                    $flashBag->add("error",
                        $texte->getOneOrCreate("emailInvalide", "Une des adresses email renseignée n'est pas valide."));
                    break;
                }
            }

            if ($flashBag->has("error") == false && !empty($emails)) {
                $tags = array();
                $tags["message"] = $this->request->request->get("message");
                $tags["link"] = substr(SITE_URL, 0, -1) . $this->generateUrl("login", array("email" => $client->client_email));
                foreach ($emails as $email) {
                    try {
                        $mailer->send("email_parrainage", array($email), $tags);
                    } catch(Exception $e){
                        $flashBag->add("error", $texte->getOneOrCreate("errSendMail",
                            "Une erreur s'est produite et a empêché l'envoi d'email, merci de ré-essayer plus tard."));
                        if(Tools::isDediServicesIp()) $flashBag->add("error", $e->getMessage());
                    }
                }
                $flashBag->add("success",
                    $texte->getOneOrCreate("parrMailSent",
                        "Les emails de parrainage ont été correctement envoyés"));
            }
        }

        return $this->redirect($this->generateUrl("compte_parrainage"));
    }

    /**
     * @Route("/address_book", name="compte_adresse", methods = { "GET" })
     */
    public function address_book()
    {
        $texte = new Texte();
        $bd = Breadcrumb::getInstance();
        $bd->add($texte->getOneOrCreate('mon_compte', 'Mon compte'));
        $bd->add($texte->getOneOrCreate('mes_adresses', 'Mes adresses'));

        $pays = new Pays();
        $client = new Client();
        $adresse = new Adresse();
        $flashBag = $this->request->getSession()->getFlashBag();
        $error = $flashBag->get('error', []);
        $success = $flashBag->get('success', []);

        $tplVars = array();
        $tplVars["pays"] = $pays->get();
        $tplVars["civilite"] = $client->getCivilite();

        $arrAdresse = $adresse->get(array("client_id" => $this->getCurrentClient()->client_id));
        $tplVars["adresse"] = $arrAdresse;

        // address asked or default to the one set as default
        if ($this->request->get('nouveau') == 1) {
            $flashBag->set('nouveau', 1);
        }

        $requestId = $this->request->get('adresse_id', null);

        if ($requestId) {
            $flashBag->get('nouveau');
        }

        $isNew = $flashBag->peek('nouveau');

        if (empty($isNew)) {
            $selectedAdress = [];
            if ($flashBag->peek('post')) {
                $selectedAdress = $flashBag->get('post');
            } else {
                if ($requestId) {
                    foreach ($tplVars["adresse"] as $adress) {
                        if ($adress['adresse_id'] == $requestId) {
                            $selectedAdress = $adress;
                        }
                    }
                } else {
                    foreach ($tplVars["adresse"] as $adress) {
                        if ($adress['adresse_default'] == 1) {
                            $selectedAdress = $adress;
                        }
                    }
                }
            }
        } else {
            $selectedAdress = $flashBag->get('post');
        }
        return $this->render('account/address_book.tpl', compact('tplVars', 'selectedAdress', 'error', 'success', 'isNew'));
    }

    /**
     * @Route("/address_book_update", name="compte_adresse_update", methods = { "POST" })
     */
    public function address_book_update()
    {
        $texte = new Texte();
        $adresse = new Adresse();
        $flashBag = $this->request->getSession()->getFlashBag();
        $params = $this->request->request->all();

        $form = new AddressForm();
        $form->add(new Field(
            "adresse_id",
            array("not_empty" => new NotBlank()),
            array("not_empty" => "Identifiant d'adresse manquant.")
        ));

        if ($error = $form->handle($this->request)) {
            $flashBag->add("error", $error);
            $flashBag->set('post', $params);
        }

        if(!$flashBag->has("error")){
            // update
            $addr = new Adresse($params["adresse_id"]);
            $addr->hydrate($params);
            $addr->client_id = $this->getCurrentClient()->client_id;
            $adresse->update($addr->toArray());

            // set default
            if ($params["adresse_default"] != 0) {
                Tools::setDefault('adresse', 'client', $this->getCurrentClient()->client_id, 1, $params['adresse_id']);
            }

            $flashBag->add("success", $texte->getOneOrCreate("succAddrChang", "Votre adresse a été modifiée."));
        }

        return $this->redirect($this->generateUrl("compte_adresse", array("adresse_id" => $params["adresse_id"])));
    }

    /**
     * @Route("/address_book_create", name="compte_adresse_create", methods = { "POST" })
     */
    public function address_book_create()
    {
        $texte = new Texte();
        $adresse = new Adresse();
        $flashBag = $this->request->getSession()->getFlashBag();
        $params = $this->request->request->all();

        $form = new AddressForm();

        if ($error = $form->handle($this->request)) {
            $flashBag->add("error", $error);
            $flashBag->set('post', $params);
        }

        if(!$flashBag->has("error")){
            // save address
            $addr = new Adresse();
            $params["client_id"] = $this->getCurrentClient()->client_id;
            $addr->hydrate($params);
            $addressId = $adresse->save($addr->toArray());

            // set default
            if (!empty($params["adresse_default"]))
                Tools::setDefault('adresse', 'client', $this->getCurrentClient()->client_id, 1, $addressId);

            $flashBag->add("success", $texte->getOneOrCreate("succAddrCrea", "Votre adresse a été créée."));
            $flashBag->get('nouveau');
        }

        return $this->redirect($this->generateUrl("compte_adresse", array("adresse_id" => $addressId)));
    }

    /**
     * @Route("/modify_id", name="modif_password", methods = { "GET" })
     */
    public function modify_password()
    {
        $texte = new Texte;
        $bd = Breadcrumb::getInstance();
        $bd->add($texte->getOneOrCreate('mon_compte', 'Mon compte'));
        $bd->add($texte->getOneOrCreate('mon_mot_de_passe', 'Mon mot de passe'));
        return $this->render('account/modify_id.tpl', $this->getFlashAssign(array("error", "success", "post")));
    }

    /**
     * @Route("/modify_id", name="modif_password_post", methods = { "POST" })
     */
    public function modify_password_post()
    {
        $client = new Client();
        $texte = new Texte();
        $flashBag = $this->request->getSession()->getFlashBag();
        $post = $this->request->request->all();
        $customer = $this->getCurrentClient();

        $form = new PasswordForm();

        if (!$client->passwordValidate($post["client_password_old"], $customer->client_password)) {
            $flashBag->add("error", $texte->getOneOrCreate("errPassActuel", "Le mot de passe actuel renseigné est invalide."));
        }

        if ($error = $form->handle($this->request)) {
            $flashBag->add("error", $error);
        }

        if(!$flashBag->has("error")){
            // update password in database
            $customer->client_password = $client->passwordEncrypt($post["client_password"]);
            try {
                $client->update($customer->toArray());
                $flashBag->add("success", $texte->getOneOrCreate("succPassChang", "Votre mot de passe a été modifié."));
            } catch (Exception $e) {
                $flashBag->add("error",
                    $texte->getOneOrCreate("err", "Un problème a empêché la mise à jour de votre mot de passe. Veuillez contacter un administrateur"));
            }
        }

        //assign filled parameters if an error occured
        if ($flashBag->has("error")) {
            $flashBag->set("post", $post);
        }

        return $this->redirect($this->generateUrl("modif_password"));
    }

    /**
     * @Route("/history", name="commande")
     */
    public function history()
    {
        $texte = new Texte;
        $bd = Breadcrumb::getInstance();
        $bd->add($texte->getOneOrCreate('mon_compte', 'Mon compte'));
        $bd->add($texte->getOneOrCreate('mes_commandes', 'Mes commandes'));

        $commande = new Commande;
        $arrCmd = $commande->get(array('client_id' => $this->getCurrentClient()->client_id));

        return $this->render('account/history.tpl', array("commandes" => $arrCmd));
    }

    /**
     * @Route(
     *      path         = "/history/{id}",
     *      name         = "commande_details",
     *      defaults     = { "id" = null },
     *      requirements = { "id" = "[0-9]+" })
     */
    public function history_id()
    {
        $id = $this->request->get('id');
        $texte = new Texte;
        $bd = Breadcrumb::getInstance();
        $bd->add($texte->getOneOrCreate('mon_compte', 'Mon compte'));
        $bd->add($texte->getOneOrCreate('mes_commandes', 'Mes commandes'), $this->generateUrl('commande'));
        $bd->add($texte->getOneOrCreate('suivi_commande_numero', 'Suivi commande n°') . ' ' . $id);

        $commande = new Commande;
        $tplVars = array();

        list($arrCommande) = $commande->get(array('commande_id' => $id, 'client_id'=> $this->getCurrentClient()->client_id));

        // reductions
        $totalReduction = 0;
        $c = new Commande($id);// todo : mixup with ^^
        $vouchers = $c->getReduction();
        if(!empty($vouchers)) {
            $totalReduction = array_sum(array_filter(Tools::extractValue("produit_prix", $vouchers)));
        }
        $this->template->assign("totalReduction", $totalReduction);

        if(empty($arrCommande)) {
            return $this->redirect($this->generateUrl('commande'));
        }

        $arrStatut = $commande->getHistory(array('commande_id' => $id));
        $arrProduit = $commande->getProduit(array('commande_id' => $id));

        // @TODO : Lien de suivi a ameliorer
        // recuperation du lien de suivi
        if(!empty($arrCommande['commande_livraison_colis'])) {
            $livraison = new Livraison;
            $arrLivraison = $livraison->get(array('livraison_code'=>$arrCommande['commande_livraison_code']));
            if(count($arrLivraison)>0)
                $lienColis=str_replace('%COLIS%',$arrCommande['commande_livraison_colis'],$arrLivraison[0]['livraison_suivi']);
            else
                $lienColis="";
        }

        if(!empty($lienColis))
            $tplVars["lienColis"] = $lienColis;

        $tplVars["commande"] = $arrCommande;
        $tplVars["statut"] = $arrStatut[0];
        $tplVars["produit"] = $arrProduit;

        return $this->render('account/history_id.tpl', compact('tplVars'));
    }

    /**
     * @Route("/sav", name="compte_sav", methods = { "GET" })
     */
    public function sav()
    {
        return $this->redirect($this->generateUrl('contact', ["origin" => "compte_sav"]));
    }

    /**
     * @Route(path = "/commande-pdf", name="compte_commande_pdf")
     */
    public function pdf()
    {
        $id = $this->request->get('commande_id');

        $commande = new Commande;
        list($arrCommande) = $commande->get(array('commande_id' => $id, 'client_id'=> $this->getCurrentClient()->client_id));
        $arrStatut = $commande->getHistory(array('commande_id' => $id));
        $arrProduit = $commande->getProduit(array('commande_id' => $id));
        $tplVars["commande"] = $arrCommande;
        $tplVars["statut"] = $arrStatut[0];
        $tplVars["produit"] = $arrProduit;
        $htmlContent = $this->render('account/commande_pdf.tpl', compact('tplVars'));
        $filename = 'United_equipement_commande_'.$id;

        return $this->renderPdf($htmlContent, $filename);
    }
}