<?php

namespace Controller;

use Listing;
use Breadcrumb;
use Boutique;
use Categorie;
use Marque;
use Gamme;
use Texte;
use Tools;
use Produit;

use Symfony\Component\Routing\Annotation\Route;

/**
 * Class ListingController
 * @package Controller
 */
class ListingController extends Controller
{


    /**
     * @Route(
     *      path         = "/recherche/{features}",
     *      name         = "recherche",
     *      defaults     = { "features" = null },
     *      requirements = { "features" = "[a-zA-Z0-9-_,]+" }
     * )
     */
    public function search()
    {
        $texte = new Texte();
        $listing = new Listing();
        $motclef = $this->request->get('motclef');
        $listing->setSearch($motclef);

        $this->template->assign('titre', $motclef);
        $this->template->assign(
            'desc',
            $texte->getOneOrCreate(
                'recherche_mots_cles',
                "Recherche de mots-clés"
            ) . ' &laquo; <h2>' . $motclef . '</h2> &raquo;'
        );

        Breadcrumb::getInstance()->add($texte->getOneOrCreate('recherche', 'Recherche'))
            ->add($motclef);

        return $this->show($listing);
    }

    /**
     * @Route(
     *      path         = "/nos-marques/{marque_nom}-{marque_id}/{features}",
     *      name         = "marque",
     *      defaults     = { "features" = null},
     *      requirements = { "marque_nom" = "[a-zA-Z0-9-_]+", "marque_id" = "[0-9]+", "features" = "[a-zA-Z0-9-_,]+"}
     * )
     */
    public function marque()
    {
        $listing = new Listing();

        $marque = new Marque($this->request->get('marque_id'));
        $listing->addMarque($marque);

        $this->template->assign('titre', $marque->marque_nom);
        $this->template->assign('desc', $marque->marque_nom_desc);

        Breadcrumb::getInstance()->add(
            $marque->marque_nom,
            $this->generateUrl(
                'marque',
                ['marque_nom' => \Tools::parseURL($marque->marque_nom), 'marque_id' => $marque->marque_id]
            )
        );

        return $this->show($listing);
    }

    /**
     * @Route(
     *      path         = "/selection/{gamme_nom}-{gamme_id}-{categorie_nom}-{categorie_id}/{features}",
     *      name         = "categorie_gamme",
     *      defaults     = { "features" = null},
     *      requirements = {
     *           "gamme_nom" = "[a-zA-Z0-9-_]+", "gamme_id" = "[0-9]+",
     *           "categorie_nom" = "[a-zA-Z0-9-_]+", "categorie_id" = "[0-9]+",
     *           "features" = "[a-zA-Z0-9-_,]+"
     *      }
     * )
     */
    public function categoryGamme()
    {
        $listing = new Listing();

        $category = new Categorie($this->request->get('categorie_id'));
        $listing->addCategorie($category);

        $gamme = new Gamme($this->request->get('gamme_id'));
        $listing->addGamme($gamme);

        $this->template->assign('titre', $gamme->gamme_nom_nom . " - " . $category->categorie_nom_nom);
        $this->template->assign('desc', $gamme->gamme_nom_desc . " - " . $category->categorie_nom_desc);

        Breadcrumb::getInstance()->add(
            $gamme->gamme_nom_nom . " - " . $category->categorie_nom_nom,
            $this->generateUrl(
                'categorie_gamme',
                [
                    'gamme_nom' => \Tools::parseURL($gamme->gamme_nom_nom),
                    'gamme_id' => $gamme->gamme_id,
                    'categorie_nom' => \Tools::parseURL($category->categorie_nom_nom),
                    'categorie_id' => $category->categorie_id,
                ]
            )
        );

        $listing->addGamme($gamme);

        return $this->show($listing);
    }

    /**
     * @Route(
     *      path         = "/selection/{gamme_nom_nom}-{gamme_id}/{features}",
     *      name         = "gamme",
     *      defaults     = { "features" = null},
     *      requirements = { "gamme_nom_nom" = "[a-zA-Z0-9-_]+", "gamme_id" = "[0-9]+", "features" = "[a-zA-Z0-9-_,]+"}
     * )
     */
    public function gamme()
    {
        $listing = new Listing();

        $gamme = new Gamme($this->request->get('gamme_id'));
        $listing->addGamme($gamme);

        $this->template->assign('titre', $gamme->gamme_nom_nom);
        $this->template->assign('desc', $gamme->gamme_nom_desc);

        Breadcrumb::getInstance()->add(
            $gamme->gamme_nom_nom,
            $this->generateUrl(
                'gamme',
                ['gamme_nom_nom' => \Tools::parseURL($gamme->gamme_nom_nom), 'gamme_id' => $gamme->gamme_id]
            )
        );

        $listing->addGamme($gamme);

        return $this->show($listing);
    }

    /**
     * @Route(
     *      path         = "/{categorie_nom}-{id}/{features}",
     *      name         = "liste_1",
     *      defaults     = { "features" = null },
     *      requirements = { "categorie_nom" = "[a-zA-Z0-9-_]+", "id" = "[0-9]+", "features" = "[a-zA-Z0-9-_,]+" }
     * )
     *
     * @return mixed
     */
    public function category()
    {
        return $this->showCategory();
    }

    /**
     * @Route(
     *      path         = "/{parent_nom}/{categorie_nom}-{id}/{features}",
     *      name         = "liste_2",
     *      defaults     = { "features" = null },
     *      requirements = { "parent_nom" = "[a-zA-Z0-9-_]+", "categorie_nom" = "[a-zA-Z0-9-_]+", "id" = "[0-9]+", "features" = "[a-zA-Z0-9-_,]+" }
     * )
     *
     * @return mixed
     */
    public function subcategory()
    {
        return $this->showCategory();
    }

    /**
     * @Route(
     *      path         = "/{grand_parent_nom}/{parent_nom}/{categorie_nom}-{id}/{features}",
     *      name         = "liste_3",
     *      defaults     = { "features" = null },
     *      requirements = { "grand_parent_nom" = "[a-zA-Z0-9-_]+", "parent_nom" = "[a-zA-Z0-9-_]+", "categorie_nom" = "[a-zA-Z0-9-_]+", "id" = "[0-9]+", "features" = "[a-zA-Z0-9-_,]+" }
     * )
     *
     * @return mixed
     */
    public function subsubcategory()
    {
        return $this->showCategory();
    }

    /**
     * Permet d'afficher un listing de produit via les catégories
     *
     * @return mixed
     */
    protected function showCategory()
    {

        $listing = new Listing();

        $categorie = new Categorie($this->request->get('id'));

        $listing->addCategorie($categorie);
        $this->template->assign('categorie', $categorie);

        $this->template->assign('titre', $categorie->categorie_nom_nom);
        $this->template->assign('desc', $categorie->categorie_nom_desc);

        $categorie->buildBreadcrumb($categorie->categorie_id);

        return $this->show($listing);
    }

    /**
     * Permet d'afficher un listing produit en fonciton du param Listing passé.
     * Permet de récupérer les produits a afficher / calculer les cara a afficher
     *
     * @param Listing $listing
     *
     * @return mixed
     *
     * @throws \DediException
     */
    public function show(Listing $listing)
    {

        $config = Boutique::getInstance()->getConfig();
        // on injecte la request dans l'objet listing pour générer les urls
        $listing->setRequest($this->request);

        $attributes = $this->request->attributes->all();
        $urlPage = $this->generateUrl(
            $attributes['_route'],
            array_diff_key($attributes, array('_route' => '', 'features' => ''))
        );
        $this->template->assign('urlPage', $urlPage);

        if ($this->request->get('features') != false) {

            $listing->setCaracteristiqueUrl($this->request->get('features'));

            $arrCaraValeur = $listing->getCaracteristiqueSelect();
            $titre = $this->template->getTemplateVars('titre');
            foreach ($arrCaraValeur as $caraVal) {
                $titre.= " " . $caraVal['caracteristique_valeur_nom_nom'];
            }
            $this->template->assign('titre', $titre);

            if (count($arrCaraValeur) >= $config['listing']['max_cara']) {
                $this->template->assign('metaRobots', 'noindex, nofollow');
            }
        }

        if ($this->request->get('page') != false) {
            $listing->setPage($this->request->get('page'));
        }

        if ($this->request->get('limit') != false) {
            $listing->setLimit($this->request->get('limit'));
        }

        if ($this->request->get('order') != false) {
            $listing->setOrder($this->request->get('order'));
        }

        if ($this->request->get('prix_min') != null && $this->request->get('prix_max') != null) {
            $param = array($this->request->get('prix_min'), $this->request->get('prix_max'));
            $paramUrl = array(
                'prix_min' => $this->request->get('prix_min'),
                'prix_max' => $this->request->get('prix_max')
            );
            $listing->setPrix($param);
            $this->template->assign('prix_min', $this->request->get('prix_min'));
            $this->template->assign('prix_max', $this->request->get('prix_max'));
            $this->template->assign('urlWithPrice', true);
            $this->template->assign('urlPrice', http_build_query($paramUrl));
        }

        $arrProd = $listing->getProductsToDisplay();

        $this->template->assign('arrProd', $arrProd);
        $this->template->assign('countProduct', $listing->getCountProducts());

        // filtres prix
        $listing->setPrice($listing->getProducts());
        if (!empty($arrProd)) {
            $this->template->assign('lepluspetitprix', $listing->getPrixMin());
            $this->template->assign('leplusgrandprix', $listing->getPrixMax());
            $this->template->assign('baseUrl', ($this->request->get('features') ?
                $this->request->get('features') :
                $urlPage
            ));
        }

        $this->template->assign('arrCaracteristique', $listing->getCaracteristique());

        $this->template->assign('arrCritere', $listing->getCaracteristiqueSelect());

        $this->template->assign('arrChildCateg', $listing->getChildCategories());

        $pagination = $listing->getPagination();
        $this->template->assign('pagination', $pagination);

        $ref = $listing->getReferencement();
        $this->template->assign('ref', $ref);

        # Ventes flash
        $gammeConfig = \Boutique::getInstance()->getConfig('gamme');
        if(isset($gammeConfig['flash_sales_id'])){
            $flash_sales_id = $gammeConfig['flash_sales_id'];
        }
        $etiquetteConfig = \Boutique::getInstance()->getConfig('etiquette');
        if(isset($etiquetteConfig['flash_sales_id'])) {
            $flash_sales_etiquette_id = $etiquetteConfig['flash_sales_id'];
        }
        $this->template->assign('gamme_flash_sales_id', $flash_sales_id);
        $this->template->assign('etiquette_flash_sales_id', $flash_sales_etiquette_id);

        return $this->render('listing.tpl');
    }

    /**
     * @Route(
     *      path         = "/listing/set_mode",
     *      name         = "liste_setmode",
     *      methods      = { "POST" }
     * )
     *
     * @return mixed
     */
    public function setModeList()
    {
        $this->getSession()->set('listmode', $this->request->get('listmode'));

        return $this->response(array('status' => true));
    }
}
