<?php

namespace Controller;

use Breadcrumb;
use Boutique;
use Categorie;
use Cms;
use Produit;
use Texte;
use Attribut;
use Tools;
use Temoignage;
use Ads;

use Symfony\Component\Routing\Annotation\Route;

/**
 * Class ProductController
 * @package Controller
 */
class ProductController extends Controller
{

    /**
     * @Route(
     *      path         = "/{produit_nom}-{id}.html",
     *      name         = "produit",
     *      requirements = { "produit_nom" = "[a-zA-Z0-9-_]+", "id" = "[0-9]+" }
     * )
     * @return mixed
     *
     * @throws \DediException
     */
    public function show()
    {

        $tpl_vars = array();

        $categorie = new Categorie();
        $produit = new \Shopbydedi\Produit();
        $txt = new Texte();
        $bd = Breadcrumb::getInstance();

        // recup fiche produit
        $prod = $produit->get(array('produit_id' => $this->request->get('id')));

        if (empty($prod)) {
            return $this->redirect($this->generateUrl('home'));
        } else {
            $prod = $prod[0];
        }

        //recup temoignage
        $temoignage = new Temoignage();
        $prod['temoignages'] = $temoignage->infos(array('produit_id' => $prod['produit_id']));

        // TEST si c'est la bonne url, si le nom du produit correspond a l'id
        $goodUrl = $this->generateUrl(
            'produit',
            ['id' => $this->request->get('id'), 'produit_nom' => Tools::parseURL($prod['produit_nom_nom'])]
        );

        if ($goodUrl != $this->request->getPathInfo()) {
            if ($this->request->getQueryString() != null) {
                $goodUrl.= '?' . $this->request->getQueryString();
            }

            return $this->redirect($goodUrl, 301);
        }
        //-- TEST si c'est la bonne url, si le nom du produit correspond a l'id

        // recup produit associé
        $config = Boutique::getInstance()->getConfig();
        $limitProdAssoc = $config['produit']['associe'];
        $arrAssoc = $produit->getAssoc(array('produit_id' => $prod['produit_id'], 'limit' => $limitProdAssoc));
        if (count($arrAssoc) < $limitProdAssoc) {

            $tmp = $produit->getSearch([
                    "categorie_id" => $prod['categorie_id'],
                    "produit_id_not_id" => $prod['produit_id'],
                    "limit" => $limitProdAssoc - count($arrAssoc),
                    'order' => 'random'
            ]);

            $arrAssoc = array_merge($arrAssoc, $tmp);
        }
        $tpl_vars['arrAssoc'] = $arrAssoc;

        // fil ariane
        $categorie->buildBreadcrumb($prod['categorie_id']);
        $bd->add($prod['produit_nom_nom']);
        //--

        // liens cms
        $cmsLinks = array();
        $pages = array(Cms::PAGE_LIVR, Cms::PAGE_PAY, Cms::PAGE_SOR);
        $cms = new Cms();
        foreach ($pages as $p) {
            list($page) = $cms->get(array("cms_page_id" => $p));
            $cmsLinks[] = $page;
        }
        $tpl_vars["cmsLinks"] = $cmsLinks;
        // --

        $paramUrl = array();

        $cat = new Categorie($prod['categorie_id']);
        $treeCat = array();
        $treeCat[] = $cat;

        do {
            $cat = new Categorie($cat->parent_id);
            $treeCat[] = $cat;
        } while (!empty($cat->parent_id));

        $treeCat = array_reverse($treeCat);

        $nbCat = count($treeCat);

        if (!empty($treeCat[$nbCat-2])) {
            $paramUrl['categorie_nom'] = $treeCat[$nbCat-2]->categorie_nom_nom;
            $paramUrl['id'] = $treeCat[$nbCat-2]->categorie_id;
        }
        if (!empty($treeCat[$nbCat-3])) {
            $paramUrl['parent_nom'] = $treeCat[$nbCat-3]->categorie_nom_nom;
        }

        $cat = new Categorie($prod["categorie_id"]); //recup parent cat name
        $tags = array_merge($prod, $cat->toArray());

        $attribut = new \Shopbydedi\Attribut();
        $produitConfig = Boutique::getInstance()->getConfig('produit');
        $produitOptionNotTailleIDs = $produitConfig['option']['not_taille_id'];

        $tailleAttributs = array();
        $couleurAttributs = array();
        $attributs = $attribut->get(array('produit_id' => $prod['produit_id']));
        if(!empty($attributs)){
            foreach($attributs as $attr){
                if(!in_array($attr['produit_option_id'], explode(',', $produitOptionNotTailleIDs))){
                    $tailleAttributs[] = $attr;
                } else {
                    $couleurAttributs[] = $attr;
                }
            }
        }
        $tpl_vars['attributsTaille'] = $tailleAttributs;
        $tpl_vars['attributsCouleur'] = $couleurAttributs;


        $ref['title'] = (!empty($prod['produit_nom_meta_title'])) ?
            $prod['produit_nom_meta_title'] : $txt->getOne('ref_title_produit', true, $tags);

        $ref['description'] = (!empty($prod['produit_nom_meta_desc'])) ?
            $prod['produit_nom_meta_desc'] : $txt->getOne('ref_desc_produit', true, $tags);

        $ref['keywords'] = (!empty($prod['produit_nom_meta_kw'])) ?
            $prod['produit_nom_meta_kw'] : $txt->getOne('ref_kw_produit', true, $tags);

        $ref['other'] = (!empty($prod['produit_nom_meta_other'])) ?
            $prod['produit_nom_meta_other'] : $txt->getOne('ref_other_produit', true, $tags);

        $gamme = new \Gamme();
        $prod['gammes'] = $gamme->get(array('produit_id' => $prod['produit_id']));

        $ref['baseline'] = $txt->getOne('baseline_produit', true, $tags);
        $tpl_vars['ref'] = $ref;

        $tpl_vars['p'] = $prod;
        $tpl_vars['fiche_produit'] = true;

        //On remonte les images du produit
        $arrImage = $produit->getImage(array('produit_id' => $prod['produit_id']));
        foreach ($arrImage as $k => $v) {
            $arrImage[$k]['image_id'] = 'img' . ($k + 1);
        }
        $this->template->assign('arrImage', $arrImage);

        # Ventes flash
        $gammeConfig = \Boutique::getInstance()->getConfig('gamme');
        if(isset($gammeConfig['flash_sales_id'])){
            $flash_sales_id = $gammeConfig['flash_sales_id'];
        }
        $etiquetteConfig = \Boutique::getInstance()->getConfig('etiquette');
        if(isset($etiquetteConfig['flash_sales_id'])) {
            $flash_sales_etiquette_id = $etiquetteConfig['flash_sales_id'];
        }
        $this->template->assign('gamme_flash_sales_id', $flash_sales_id);
        $this->template->assign('etiquette_flash_sales_id', $flash_sales_etiquette_id);

        return $this->render('produit.tpl', $tpl_vars);
    }

    /**
     * @Route(
     *      path         = "/product/select-option",
     *      name         = "select_option",
     *      methods      = { "POST" }
     * )
     *
     */
    public function selectOption()
    {
        $povId = $this->request->get('produit_option_valeur_id');
        $povIdBis = $this->request->get('produit_option_valeur_id_bis');

        $boutique = Boutique::getInstance();

        $json = array();

        if (!empty($povId) || !empty($povIdBis)) {
            $attribut = new Attribut();
            if (!empty($povId) && !empty($povIdBis)) {
                $json['total'] = 0;
                $param = array();
                $param['produit_id'] = $this->request->get('produit_id');
                $param['produit_option_valeur_id'] = $povId;
                $param['produit_option_valeur_id_bis'] = $povIdBis;
                $att = $attribut->getBloc($param);
                $json['att_id'] = $att[0]['produit_attribut_id'];
                $json['image'] = '';
                $json['total'] = $att[0]['attribut_produit_prix_ttc'];

                $json['total'] = $boutique->getDevise()->format($json['total']);
            } elseif (!empty($povIdBis['produit_option_valeur_id_bis'])) {
                // TODO : change image
                $json['image'] = '';
            }

        }

        return $this->response($json);
    }

    /**
     * @Route(
     *      path         = "/product/select-quantity",
     *      name         = "select_quantity",
     *      methods      = { "POST" }
     * )
     *
     */
    public function selectQuantity()
    {

        $json = array();

        $attribut = new Attribut();
        $result = $attribut->get([
            'produit_id' => $this->request->get('produit_id'),
            'produit_attribut_conditionnement' => array($this->request->get('quantity')),
            'order' => 'conditionnement'
        ]);

        if (!empty($result)) {
            $json['att_id'] = $result[0]['produit_attribut_id'];
            $json['total-remise'] = $result[0]['attribut_prix_ttc'] * $this->request->get('quantity');
            $json['total'] = $result[0]['attribut_produit_prix_ttc'] * $this->request->get('quantity');

            $devise = Boutique::getInstance()->getDevise();
            $json['total-remise'] = $devise->format($json['total-remise']);
            $json['total'] = $devise->format($json['total']);
        }

        return $this->response($json);
    }
}
