<?php

/*
 * Exemple pour création export CSV
  $file = SITE_DIR . 'dir/my_file.csv';
  $csv = new Csv($file);
  $data = array('str 1', 'str 2', 'str 3', 'str 4');
  $csv->writeData($data);
  $csv->getCsv();

  OU
  $file = SITE_DIR . 'dir/my_file.csv';
  Csv::getCsvFromArray($file, $data);

 * Exemple pour lecture CSV
  $file = SITE_DIR . 'dir/my_file.csv';
  $csv = new Csv($file);
  $csv->loadData();
  while ($data = $csv->getNextLine()) {
  // action data[0], data[1] ...
  }

 */

/**
 * Class Csv
 */
class Csv {

    public $file;
    private $_myData;
    private $_fd;
    private $_compliantMsExcel;

    public function __construct($file = '') {
        $this->file = $file;
        $this->_myData = array();
        $this->_fd = fopen($this->file, "c+");
        $this->_compliantMsExcel = false;
    }

    static public function getCsvFromArray($file, $dataArray, $getFile = true) {
        if (!empty($file) && !empty($dataArray) && is_array($dataArray)) {
            $tabCsv = new Csv($file);
            $tabCsv->setCompliantMsExcel(true);
            $tabCsv->writeData(array_keys($dataArray[0]));
            foreach ($dataArray as $da) {
                $tabCsv->writeData($da);
            }
            if ($getFile == true)
                $tabCsv->getCsv(true);
        } else {
            return false;
        }
    }

    static public function getCsvFromSQL($file, $sql, $getFile = true) {
        if (!empty($file) && !empty($sql)) {
            $results = SiteSQL::getInstance()->getAll($sql);
            if (!empty($results)) {
                self::getCsvFromArray($file, $results, $getFile);
            }
        } else {
            return false;
        }
    }

    public function loadData() {
        if (!$this->_fd) {
            return false;
        } else {
            while (($data = fgetcsv($this->_fd, 10000, ";")) !== FALSE) {
                $this->_myData[] = $data;
            }
            return (count($this->_myData) > 0);
        }
    }

    public function getNextLine() {
        return array_shift($this->_myData);
    }

    public function writeData($data = array()) {
        if ($this->_compliantMsExcel == true) {
            array_walk($data, create_function('&$val', '$val = utf8_decode($val);'));
        }
        array_walk($data, create_function('&$val', '$val = str_replace(\'"\', \'\', $val);'));
        array_walk($data, create_function('&$val', '$val = html_entity_decode($val);'));
        fputcsv($this->_fd, $data, ';', '"');
    }

    public function setCompliantMsExcel($val) {
        $this->_compliantMsExcel = $val;
    }

    public function getNbLine() {
        return count($this->_myData);
    }

    public function getArrayData() {
        return $this->_myData;
    }

    public function setFormat($format, $str) {
        switch ($format) {
            case 'price':
                $str = str_replace('.', ',', $str);
                break;
        }
        return $str;
    }

    // parametre delete à true => permet de supprimer le fichier après le téléchargement
    public function getCsv($delete = false) {
        header("Content-disposition: attachment; filename=" . basename($this->file));
        header('Content-Type: application/octet-stream');
        header('Content-Transfer-Encoding: binary');
        header("Content-Length: " . filesize($this->file));
        header('Expires: 0');
        header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
        header('Pragma: public');
        flush();
        readfile($this->file);
        if ($delete)
            unlink($this->file);
        exit;
    }

}