<?php

use Monolog\Logger;
use Monolog\Handler\StreamHandler;
use Monolog\Formatter\JsonFormatter;

/**
 * DediException class
 *
 * Exception à lancer.
 * Si debug activer : ajoute l'exception dans la debug bar PHP
 * Si non, ajoute l'excption aux logs
 */
class DediException extends Exception
{
    /**
     * Constructeur
     *
     * @param null $message
     * @param int $code
     * @param Exception $previous
     *
     * @return void
     */
    public function __construct($message = null, $code = 0, Exception $previous = null)
    {
        parent::__construct($message, $code, $previous);

        $dbg = Application::getDebugBar();
        if (getenv('APP_DEBUG') && !empty($dbg)) {
            // ajout exception a la debugbar
            $dbg['exceptions']->addException($this);
        } else {
            // sinon dans les logs

            // @TODO : mettre dans une classe a part => on peut changer le stockage quand on veut
            $logger = new Logger('dedi_exception');

            if (false == is_dir(LOGS_DIR)) {
                mkdir(LOGS_DIR, 0775, true);
            }

            $stream = new StreamHandler(LOGS_DIR . 'app.log', Logger::ERROR);
            $stream->setFormatter(new JsonFormatter());
            $logger->pushHandler($stream);
            $logger->addError($this->getMessage(), get_object_vars($this));
        }
    }
}
