<?php
/**
 * Router class file
 *
 * PHP Version 5.3
 *
 * @category Dedi
 * @package  Dedi
 * @author   Matthieu Bouquet <matthieu@dedi-agency.com>
 * @license  http://opensource.org/licenses/gpl-license.php GNU Public License
 * @link     http://www.dedi-agency.com
 */

use Symfony\Component\Routing\Router as RouterSF;
use Symfony\Component\Routing\RequestContext;
use Symfony\Component\Config\FileLocator;

use Symfony\Component\Routing\Loader\AnnotationDirectoryLoader;
use Annotation\AnnotationClassLoader;
use Doctrine\Common\Annotations\AnnotationReader;
use Doctrine\Common\Annotations\CachedReader;
use Doctrine\Common\Cache\ApcCache;

/**
 * Router class
 *
 * Permet de faire toutes les opérations courantes relative aux commandes
 *
 * PHP Version 5.3
 *
 * @category Dedi
 * @package  Dedi
 * @author   Matthieu Bouquet <matthieu@dedi-agency.com>
 * @license  http://opensource.org/licenses/gpl-license.php GNU Public License
 * @link     http://www.dedi-agency.com
 *
 */
class Router
{
    /**
     * @var \Symfony\Component\Routing\Router
     */
    protected $router;

    /**
     * @var \Annotation\AnnotationClassLoader
     */
    protected $annotationClassLoader;

    /**
     * @var \Symfony\Component\Routing\Loader\AnnotationDirectoryLoader
     */
    protected $annotationDirectoryLoader;

    /**
     * constructeur
     */
    public function __construct()
    {
        $locator = new FileLocator(array(__DIR__ . '/../..'));
        $requestUri = (!empty($_SERVER['REQUEST_URI'])) ? $_SERVER['REQUEST_URI'] : '';
        $requestContext = new RequestContext($requestUri);

        // $cache = new ApcCache();
        $cache = new \Doctrine\Common\Cache\ArrayCache();

        $reader = new CachedReader(
            new AnnotationReader(),
            $cache,
            $debug = true
        );

        $this->annotationClassLoader = new AnnotationClassLoader($reader);
        $this->annotationDirectoryLoader = new AnnotationDirectoryLoader($locator, $this->annotationClassLoader);

        $this->router = new RouterSF(
            $this->annotationDirectoryLoader,
            'src/Dedi/Controller',
            array(),
            $requestContext
        );
    }

    /**
     * retourne le Router
     *
     * @return \Symfony\Component\Routing\Router
     */
    public function getRouter()
    {
        return $this->router;
    }

    /**
     * @return \Annotation\AnnotationClassLoader
     */
    public function getAnnotationClassLoader()
    {
        return $this->annotationClassLoader;
    }

    /**
     * @return \Symfony\Component\Routing\Loader\AnnotationDirectoryLoader
     */
    public function getAnnotationDirectoryLoader()
    {
        return $this->annotationDirectoryLoader;
    }
}