<?php

/**
 * https://preprod-admin.paybox.com
 * 199988832 / 1999888I
 *
 * configuration paybox
 * $config['paybox']['site'] = '1999888';
 * $config['paybox']['rang'] = '32';
 * $config['paybox']['identifiant'] = '1686319';
 * $config['paybox']['key'] = '1999888I' | '0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF';
 * $config['paybox']['mode'] = 'dev'|'prod'
 *
 * VERSION : 00103 pour Paybox Direct
             00104 pour Paybox Direct Plus
 */

namespace Shopbydedi\Payment;

use Millesimes\PaiementComplement;
use Tpe;
use Commande;
use Panier;

/**
 * Class Paybox
 * @package Shopbydedi\Payment
 */
class PayboxDirect implements PaymentDirectInterface
{
    protected $site;
    protected $rang;
    protected $identifiant;
    protected $key;
    protected $servers;

    /**
     * @param $config
     *
     * @throws \DediException
     */
    public function __construct($config)
    {

        if (empty($config['paybox'])) {
            throw new \DediException('configure infos paybox in boutique config');
        }

        $this->site = $config['paybox']['site'];
        $this->rang = $config['paybox']['rang'];
        $this->identifiant = $config['paybox']['identifiant'];
        $this->key = $config['paybox']['key'];

        if ($config['paybox']['mode'] == 'dev') {
            $this->servers  = ['https://preprod-ppps.paybox.com/PPPS.php'];
        } else {
            $this->servers = [
                'https://ppps.paybox.com/PPPS.php',
                'https://ppps1.paybox.com/PPPS.php',
            ];
        }
    }

    /**
     * demande d'authorisation sur le compte, pour voir si l'argent peut etre prelevé.
     * Le débit se fera ulterieurement
     *
     * @param Tpe $tpe
     * @param $amount
     * @param $cardNumber
     * @param $cardExpirationDate
     * @param $cardCrypto
     *
     * @return mixed
     */
    public function doAuthorization(Tpe $tpe, $amount, $cardNumber, $cardExpirationDate, $cardCrypto)
    {
        $params = array();
        $params['VERSION'] = '00104';
        $params['TYPE'] = '00001';
        $params['SITE'] = $this->site;
        $params['RANG'] = $this->rang;
        $params['CLE'] = $this->key;
        $params['NUMQUESTION'] = $tpe->tpe_reference_id;
        $params['MONTANT'] = round($amount * 100);
        $params['DEVISE'] = '978';
        $params['REFERENCE'] = $tpe->tpe_reference_id;
        $params['PORTEUR'] = $cardNumber;
        $params['DATEVAL'] = $cardExpirationDate;
        $params['CVV'] = $cardCrypto;
        $params['ACTIVITE'] = '024';
        $params['DATEQ'] = date('dmYHis');
        $params['PAYS'] = '';

        $response = $this->sendRequest($params);

        return $this->response($response);
    }

    /**
     * demande de débit sur le compte. Debite au jour j le compte
     *
     * @param Tpe $tpe
     * @param $amount
     * @param $cardNumber
     * @param $cardExpirationDate
     * @param $cardCrypto
     *
     * @return mixed
     */
    public function doDebit(Tpe $tpe, $amount, $cardNumber, $cardExpirationDate, $cardCrypto)
    {
        $params = array();
        $params['VERSION'] = '00104';
        $params['TYPE'] = '00002';
        $params['SITE'] = $this->site;
        $params['RANG'] = $this->rang;
        $params['CLE'] = $this->key;
        $params['NUMQUESTION'] = $tpe->tpe_reference_id;
        $params['MONTANT'] = round($amount * 100);
        $params['DEVISE'] = '978';
        $params['REFERENCE'] = $tpe->tpe_reference_id;
        $params['PORTEUR'] = $cardNumber;
        $params['DATEVAL'] = $cardExpirationDate;
        $params['CVV'] = $cardCrypto;
        $params['ACTIVITE'] = '024';
        $params['DATEQ'] = date('dmYHis');
        $params['PAYS'] = '';

        $response = $this->sendRequest($params);

        return $this->response($response);
    }

    /**
     * @param $params
     *
     * @return array
     */
    public function sendRequest($params)
    {
        // Set the curl parameters.
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->servers[0]);
        curl_setopt($ch, CURLOPT_VERBOSE, 1);

        // Turn off the server and peer verification (TrustManager Concept).
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);

        // Set the request as a POST FIELD for curl.
        $params = array_map('urlencode', $params);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));

        // Get response from the server.
        $httpResponse = curl_exec($ch);

        if(!$httpResponse) {
            exit("failed: ".curl_error($ch).'('.curl_errno($ch).')');
        }

        // Extract the response details.
        $httpResponseAr = explode("&", $httpResponse);

        $httpParsedResponseAr = array();
        foreach ($httpResponseAr as $i => $value) {
            $tmpAr = explode("=", $value);
            if(sizeof($tmpAr) > 1) {
                $tmpAr[1] = urldecode($tmpAr[1]);

                $httpParsedResponseAr[$tmpAr[0]] = $tmpAr[1];
            }
        }

        return $httpParsedResponseAr;
    }

    /**
     * @param array $request
     * @throws \DediException
     *
     * @return mixed
     */
    function response($request)
    {

        $return = ['status' => false];

        if (!empty($request['NUMQUESTION'])) {

            /**
             * L'execution est ok on traite les cas
             * 00 Transaction approuvee ou traitee avec succes
             */

            $info = '';
            foreach ($request as $k => $v) {
                $info .= $k . ' = ' . $v . "<br/>";
            }

            if ($request['CODEREPONSE'] == "00000") {

                $tpe = new Tpe($request['NUMQUESTION']);

                if ($tpe->tpe_reference_type == 'commande') {
                    $commande = new Commande();
                    try {
                        $id = $commande->saveFromPanier(Panier::getInstance());
                        $tpe->assoc($request['NUMQUESTION'], $id);
                    } catch (\Exception $e) {

                    }
                } elseif ($tpe->tpe_reference_type == 'complement') {
                    /** * @var \Millesimes\PaiementComplement $paiementComplement */
                    $paiementComplement = unserialize($tpe->tpe_reference_session);
                    $paiementComplement->valid();

                    // on associe la commande et on rempli les infos de retour a la reference
                    $tpe->assocComplement($request['NUMQUESTION'], $paiementComplement->paiement_complement_id);
                }

                $tpe->addResponse($request['NUMQUESTION'], $request);

                $return['status'] = true;
                $return['tpe_reference_id'] = $request['NUMQUESTION'];

            } else {

                $tpe = new Tpe;
                $tpe->addResponse($request['NUMQUESTION'], $request);

                $return = array_merge($request, $request);
            }
        }

        return $return;
    }
}
