<?php

namespace Shopbydedi\Payment;

use Exception;
use Panier;
use PayPal\Api\PaymentExecution;
use PayPal\Auth\OAuthTokenCredential;
use PayPal\Rest\ApiContext;
use Boutique;
use Application;

use PayPal\Api\Amount;
use PayPal\Api\Item;
use PayPal\Api\ItemList;
use PayPal\Api\Payer;
use PayPal\Api\Payment;
use PayPal\Api\RedirectUrls;
use PayPal\Api\Transaction;
use Tpe;

/**
 * Class PaypalExpress
 * @package Shopbydedi\Payment
 */
class PaypalExpress implements PaymentInterface
{

    protected $clientId;
    protected $clientSecret;

    /**
     * @var \Router
     */
    protected $router;

    /**
     * @param $config
     */
    public function __construct($config)
    {
        $this->clientId = $config['paypal']['client_id'];
        $this->clientSecret = $config['paypal']['client_secret'];

        $this->router = Application::getRouter();
    }

    /**
     * @param Tpe $tpe
     * @param float $amount
     * @param array $params
     *
     * @return mixed
     */
    public function request(Tpe $tpe, $amount, $params = array())
    {

        $amt = new Amount();
        $amt->setCurrency("EUR")
            ->setTotal($amount);

        $transaction = new Transaction();
        $transaction->setAmount($amt)
            ->setDescription("Payment description")
            ->setInvoiceNumber(uniqid());

        return $this->processTransaction($transaction);
    }

    /**
     * @param Panier $panier
     * @param float $amount
     * @param array $params
     *
     * @return null|string
     */
    public function requestFromPanier(Panier $panier, $amount, $params = array())
    {

        $caddie = $panier->getProducts();

        $items = [];
        foreach ($caddie as $cad) {
            $item = (new Item())
                ->setName($cad['produit_nom_nom'])
                ->setCurrency('EUR')
                ->setQuantity($cad['quantite_order'])
                ->setPrice($cad['produit_total_ttc']);
            $items[] = $item;
        }

        $itemList = (new ItemList())->setItems($items);

        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();
        $shipping = $panier->getShipping();

        /*$details = new Details();
        $details->setShipping($shipping->port_ttc)
            ->setTax($aPayerTTC - $aPayerHT)
            ->setSubtotal($aPayerTTC - $shipping->port_ttc);*/

        $amount = new Amount();
        $amount->setCurrency("EUR")
            ->setTotal($aPayerTTC);
        // ->setDetails($details);

        $transaction = new Transaction();
        $transaction->setAmount($amount)
            // ->setItemList($itemList)
            ->setDescription("Payment description")
            ->setInvoiceNumber(uniqid());

        return $this->processTransaction($transaction);
    }

    /**
     * @param Transaction $transaction
     *
     * @return null|string
     */
    protected function processTransaction(Transaction $transaction)
    {
        $payer = new Payer();
        $payer->setPaymentMethod("paypal");

        $redirectUrls = new RedirectUrls();
        $redirectUrls
            ->setReturnUrl(substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate('paypal_express_login', ['_locale' => Boutique::getInstance()->getLangue()->ini]))
            ->setCancelUrl(substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate('panier', ['_locale' => Boutique::getInstance()->getLangue()->ini]));

        $payment = new Payment();
        $payment->setIntent("sale")
            ->setPayer($payer)
            ->setRedirectUrls($redirectUrls)
            ->setTransactions(array($transaction));

        // try {
        $payment->create($this->getApiContext());
        /*} catch (\Exception $ex) {
            // NOTE: PLEASE DO NOT USE RESULTPRINTER CLASS IN YOUR ORIGINAL CODE. FOR SAMPLE ONLY
            // ResultPrinter::printError("Created Payment Using PayPal. Please visit the URL to Approve.", "Payment", null, $request, $ex);
            // exit(1);
        }*/

        return $payment->getApprovalLink();
    }

    /**
     * @param $paymentId
     *
     * @return Payment
     */
    public function getPaymentDetails($paymentId)
    {
        return Payment::get($paymentId, $this->getApiContext());
    }

    /**
     * @param array $request
     *
     * @return Payment
     */
    public function response($request)
    {
        $apiContext = $this->getApiContext();
        // Get the payment Object by passing paymentId
        // payment id was previously stored in session in
        // CreatePaymentUsingPayPal.php
        $paymentId = $request['paymentId'];
        $payment = Payment::get($paymentId, $apiContext);

        // ### Payment Execute
        // PaymentExecution object includes information necessary
        // to execute a PayPal account payment.
        // The payer_id is added to the request query parameters
        // when the user is redirected from paypal back to your site
        $execution = new PaymentExecution();
        $execution->setPayerId($request['PayerID']);

        try {
            // Execute the payment
            $result = $payment->execute($execution, $apiContext);

            // ResultPrinter::printResult("Executed Payment", "Payment", $payment->getId(), $execution, $result);

            try {
                $payment = Payment::get($paymentId, $apiContext);
            } catch (\Exception $ex) {
                // ResultPrinter::printError("Get Payment", "Payment", null, null, $ex);
                exit(1);
            }
        } catch (\Exception $ex) {
            // ResultPrinter::printError("Executed Payment", "Payment", null, null, $ex);
            exit(1);
        }

        // ResultPrinter::printResult("Get Payment", "Payment", $payment->getId(), null, $payment);

        return $payment;
    }

    /**
     * @return ApiContext
     */
    protected function getApiContext()
    {
        // ### Api context
        // Use an ApiContext object to authenticate
        // API calls. The clientId and clientSecret for the
        // OAuthTokenCredential class can be retrieved from
        // developer.paypal.com

        $apiContext = new ApiContext(
            new OAuthTokenCredential(
                $this->clientId,
                $this->clientSecret
            )
        );

        // Comment this line out and uncomment the PP_CONFIG_PATH
        // 'define' block if you want to use static file
        // based configuration

        $apiContext->setConfig(
            array(
                'mode' => 'sandbox',
                'log.LogEnabled' => true,
                'log.FileName' => '../PayPal.log',
                'log.LogLevel' => 'DEBUG', // PLEASE USE `FINE` LEVEL FOR LOGGING IN LIVE ENVIRONMENTS
                'validation.level' => 'log',
                'cache.enabled' => true,
                // 'http.CURLOPT_CONNECTTIMEOUT' => 30
                // 'http.headers.PayPal-Partner-Attribution-Id' => '123123123'
            )
        );

        return $apiContext;
    }
}