<?php

namespace Shopbydedi;

use Model;
use Boutique;
use DediException;
use Caracteristique;
use Categorie;
use CaracteristiqueValeur;
use Langue;
use Client;

/**
 * Class Produit
 * @package Shopbydedi
 *
 * @property $produit_id
 * @property $categorie_id
 * @property $produit_type_id
 */

class Produit extends Model
{

    const TYPE_NORMAL = 1;
    const TYPE_PRIMEUR = 2;

    public static $idTaille = 1;
    public static $idCouleur = 2;

    /**
     * @var \Transformer\TransformerInterface[]
     */
    protected static $transformers = ['ProductTransformer'];

    protected $table = "produit";

    /**
     * @param null $id
     * @throws \Exception
     */
    public function __construct($id = null)
    {
        parent::__construct($id);
    }

    /**
     * @param array $formvars
     *
     * @return array
     */
    public function getOne($formvars = array())
    {
        $result = $this->get($formvars);
        return (!empty($result)) ? $result[0] : array();
    }

    /**
     * @deprecated
     *
     * @param array $formvars
     * @param string $champs
     *
     * @return array
     */
    public function getBloc($formvars = array(), $champs = "")
    {
        return $this->get($formvars, $champs);
    }

    /**
     * @param array $formvars
     *
     * @return array
     *
     * @throws DediException
     */
    public function get($formvars = array())
    {
        $lgId = Boutique::getInstance()->getLangue()->id;

        $sql = "SELECT p.produit_id, patt.produit_attribut_id, pn.produit_nom_nom, pn.produit_nom_desc";
        $sql.=", p.produit_ref_frs, p.produit_ref, p.produit_unite";
        $sql.=", pi.produit_image, m.marque_id, m.marque_nom, pc.categorie_id";
        $sql .= ", g.gamme_image, g.gamme_id, ppb.etiquette_id, en.etiquette_nom_nom, en.etiquette_nom_image_liste";
        $sql.=", COUNT(patt.produit_attribut_id) as nbattr";

        $sql.=", p.produit_prix";
        $sql.=", MIN(patt.produit_attribut_prix) AS produit_attribut_prix";
        $sql.=", MIN(ppb.produit_prix_barre_prix) AS produit_prix_barre_prix";
        $sql.=", MIN(ppb_att.produit_prix_barre_prix) AS produit_attribut_prix_barre_prix";

        $sql.=", ppb.produit_prix_barre_id AS promo";
        $sql.=", ROUND(MAX(ppb.produit_prix_barre_percent)) AS promo_percent";
        $sql.=", MAX(ppb.produit_prix_barre_valeur) AS produit_prix_barre_valeur";
        $sql.=", MAX(ppb_att.produit_prix_barre_valeur) AS produit_attribut_prix_barre_valeur";
        $sql.=", ppb.produit_prix_barre_date_fin";

        $sql.=", t.taxe_taux, p.produit_achat";
        //-- remonter des prix
        // info de stock
        if (empty($formvars['produit_quantite'])) {
            $formvars['produit_quantite'] = 1;
        }
        $sql.=", IF(p.produit_quantite-" . $formvars['produit_quantite'] . "<0,IF(p.produit_livraison,p.produit_livraison,IF(f.fournisseur_livraison,f.fournisseur_livraison," . DELAI_PAR_DEFAUT . ")),0) as delai_livraison";
        $sql.=", IF(p.produit_appro_id=0 && p.produit_quantite<=0, 1, 0) as epuise";
        $sql.=", IF(patt.produit_attribut_id, SUM(patt.produit_appro_id), p.produit_appro_id) as prod_appro";
        $sql.=", IF(patt.produit_attribut_id,SUM(IF(patt.produit_attribut_quantite>0,patt.produit_attribut_quantite,0)),p.produit_quantite) as prod_qte";
        $sql.=", patt.produit_attribut_quantite";
        $sql.=", p.produit_quantite";
        //-- info de stock

        $sql.=", IF(patt.produit_attribut_id, patt.produit_attribut_poids, p.produit_poids) as prod_poids";
        // info type produit
        $sql.=", p.produit_type_id, pt.produit_type_stock, pt.produit_type_port, pt.produit_type_class";

        $sql.=" FROM (produit p)";

        $sql.=" INNER JOIN produit_boutique pb ON(p.produit_id = pb.produit_id AND pb.boutique_id = '".Boutique::getInstance()->id."')";
        $sql.=" INNER JOIN produit_nom pn ON(p.produit_id = pn.produit_id AND pn.langue_id = '" . $lgId . "')";
        $sql.=" INNER JOIN taxe t ON(p.taxe_id = t.taxe_id)";

        $statutAtt = 'produit_attribut_statut_id = 1';
        if (!empty($formvars['statut'])) {
            $statutAtt = 'produit_attribut_statut_id IN('.implode(',', $formvars['statut']).')';
        }
        $sql.=" LEFT JOIN produit_attribut patt ON (patt.produit_id = p.produit_id AND ".$statutAtt.")";
        $sql.=" LEFT JOIN produit_image pi ON (pi.produit_id = p.produit_id AND produit_image_default='1')";
        $sql.=" LEFT JOIN marque m ON (m.marque_id = p.marque_id)";
        $sql.=" LEFT JOIN fournisseur f ON (f.fournisseur_id = p.fournisseur_id)";
        $sql.=" LEFT JOIN produit_prix_barre ppb ON (p.produit_id = ppb.produit_id AND ppb.produit_attribut_id IS NULL AND ppb.produit_prix_barre_date_debut <= NOW() AND ppb.produit_prix_barre_date_fin >= NOW() AND ppb.boutique_id = '".Boutique::getInstance()->id."')";
        $sql.=" LEFT JOIN produit_prix_barre ppb_att ON (p.produit_id = ppb_att.produit_id AND ppb_att.produit_attribut_id = patt.produit_attribut_id AND ppb_att.produit_prix_barre_date_debut <= NOW() AND ppb_att.produit_prix_barre_date_fin >= NOW() AND ppb_att.boutique_id = '".Boutique::getInstance()->id."')";
        $sql.=" LEFT JOIN produit_gamme pg ON (pg.produit_id = p.produit_id)";
        $sql.=" LEFT JOIN gamme g ON (g.gamme_id = pg.gamme_id)";
        $sql.=" LEFT JOIN etiquette_nom en ON (ppb.etiquette_id = en.etiquette_id AND en.langue_id = '" . $lgId . "')";
        $sql.=" LEFT JOIN produit_type pt ON (pt.produit_type_id = p.produit_type_id)";
        $sql.=" LEFT JOIN produit_categorie pc ON (pc.produit_id = p.produit_id)";

        $sql.=" WHERE 1";
        if (!empty($formvars['statut'])) {
            $sql.=" AND p.produit_statut_id IN (".implode(',', $formvars['statut']).")";
            $sql.=" AND pb.produit_boutique_statut_id IN (".implode(',', $formvars['statut']).")";
        } else {
            $sql.=" AND p.produit_statut_id = 1";
            $sql.=" AND patt.produit_attribut_statut_id = 1";
            $sql.=" AND pb.produit_boutique_statut_id = 1";
        }

        if (!empty($formvars['produit_id'])) {
            $sql .= " AND p.produit_id = '" . $formvars['produit_id'] . "'";
        }
        if (!empty($formvars['p.produit_id'])) {
            $sql .= " AND p.produit_id = '" . $formvars['p.produit_id'] . "'";
        }
        if (!empty($formvars['produit_attribut_id'])) {
            $sql .= " AND patt.produit_attribut_id = '" . $formvars['produit_attribut_id'] . "'";
        }

        if (!empty($formvars['produit_ref_frs'])) {
            $sql .= " AND p.produit_ref_frs = '" . $formvars['produit_ref_frs'] . "'";
        }

        if (!empty($formvars['fournisseur_id'])) {
            $sql .= " AND p.fournisseur_id = '" . $formvars['fournisseur_id'] . "'";
        }

        if (!empty($formvars['marque_id'])) {
            $sql .= " AND p.marque_id = '" . $formvars['marque_id'] . "'";
        }

        if (!empty($formvars['produit_nom_nom'])) {
            $sql .= " AND pn.produit_nom_nom LIKE '%" . $formvars['produit_nom_nom'] . "%'";
        }

        // Group By
        if (empty($formvars['group'])) {
            $formvars['group'] = '';
        }
        switch ($formvars['group']) {
            case "produit_attribut_id":
                $sql .= " GROUP BY patt.produit_attribut_id";
                break;
            default:
                $sql .= " GROUP BY p.produit_id";
                break;
        }
        //-- Group By
        // Order By
        if (!empty($formvars['order'])) {
            switch ($formvars['order']) {
                case "random":
                    $sql .= " ORDER BY RAND()";
                    break;
                case "rang":
                    $sql .= " ORDER BY p.produit_rang";
                    break;
            }
        }
        //-- Order By
        // Gestion de la limit
        if (!empty($formvars['limit'])) {

            if (!empty($formvars['page']) && $formvars['page'] != "all") {
                $offset = ($formvars['page'] - 1) * $formvars['limit'];
                $sql.=" LIMIT " . $offset . ", " . $formvars['limit'];
            } elseif (empty($formvars['page'])) {
                $sql.=" LIMIT " . $formvars['limit'];
            }
        }

        $result = $this->sql->getAll($sql);

        $this->applyTranformers($result);

        return $result;
    }

    /**
     *
     * TODO : modif remonté prix comme pour getPalierPrice
     *
     * @param array $formvars
     *
     * @return array
     */
    public function getSearch($formvars = array())
    {

        $lgId = Boutique::getInstance()->getLangue()->id;

        $where = '';

        $sql = "SELECT p.produit_id, patt.produit_attribut_id, p.marque_id, pg.gamme_id";
        if (!empty($formvars['motclef']) || !empty($formvars['search'])) {
            $sql.=", pn.produit_nom_nom";
        }

        // info stock
        $sql.=", IF(patt.produit_attribut_id, SUM(patt.produit_appro_id), p.produit_appro_id) as prod_appro";
        $sql.=", IF(patt.produit_attribut_id,SUM(IF(patt.produit_attribut_quantite>0,patt.produit_attribut_quantite,0)),p.produit_quantite) as prod_qte";
        //--info stock

        // TODO : revoir calcul prix pour intégrer le prix par groupe
        $sql.=", ROUND(IF(ppb.produit_prix_barre_id,MIN(ppb.produit_prix_barre_prix)+IF(ppb_att.produit_prix_barre_id,MIN(ppb_att.produit_prix_barre_prix),0),p.produit_prix+IF(patt.produit_attribut_id,MIN(patt.produit_attribut_prix),0)) * (1 + taxe_taux),2) AS produit_prix_ttc";

        $sql .= " FROM (produit p)";

        $sql.=" INNER JOIN produit_boutique pb ON(p.produit_id = pb.produit_id AND pb.boutique_id = '".Boutique::getInstance()->id."')";
        $sql.=" INNER JOIN produit_nom pn ON(p.produit_id = pn.produit_id AND pn.langue_id = '" . $lgId . "')";
        $sql.=" INNER JOIN taxe t ON(p.taxe_id = t.taxe_id)";

        $sql .= " LEFT JOIN produit_attribut patt ON (patt.produit_id = p.produit_id AND produit_attribut_statut_id = 1)";
        $sql .= " LEFT JOIN produit_prix_barre ppb ON (p.produit_id = ppb.produit_id AND ppb.produit_attribut_id IS NULL AND ppb.produit_prix_barre_date_debut <= NOW() AND ppb.produit_prix_barre_date_fin >= NOW() AND ppb.boutique_id = '".Boutique::getInstance()->id."')";
        if (!empty($formvars['produit_attribut_id'])) {
            $sql.=" LEFT JOIN produit_prix_barre ppb_att ON (p.produit_id = ppb_att.produit_id AND ppb_att.produit_attribut_id = patt.produit_attribut_id AND ppb_att.produit_prix_barre_date_debut <= NOW() AND ppb_att.produit_prix_barre_date_fin >= NOW() AND ppb_att.boutique_id = '".Boutique::getInstance()->id."')";
        } else {
            $sql.=" LEFT JOIN produit_prix_barre ppb_att ON (p.produit_id = ppb_att.produit_id AND ppb_att.produit_attribut_id IS NOT NULL AND ppb_att.produit_prix_barre_date_debut <= NOW() AND ppb_att.produit_prix_barre_date_fin >= NOW() AND ppb_att.boutique_id = '".Boutique::getInstance()->id."')";
        }
        $sql .= " LEFT JOIN produit_gamme pg ON(p.produit_id = pg.produit_id)";

        if (!empty($formvars['motclef'])) {
            $sql .= " LEFT JOIN marque m ON(p.marque_id = m.marque_id)";
        }

        if (!empty($formvars['categorie_id'])) {
            $sql .= " LEFT JOIN produit_categorie pc ON(p.produit_id = pc.produit_id)";
            $sql .= " LEFT JOIN categorie cat ON(cat.categorie_id = pc.categorie_id)";
            $sql .= " LEFT JOIN categorie catpar ON(catpar.categorie_id = cat.parent_id)";
            $where .= " AND (pc.categorie_id = '" . $formvars['categorie_id'] . "' OR cat.parent_id = '" . $formvars['categorie_id'] . "' OR catpar.parent_id = '" . $formvars['categorie_id'] . "') ";
        }

        if (!empty($formvars['parent_id'])) {
            if (empty($formvars['categorie_id'])) {
                $sql .= " LEFT JOIN produit_categorie pc ON(p.produit_id = pc.produit_id)";
                $sql .= " LEFT JOIN categorie cat ON(cat.parent_id = " . $formvars['parent_id'] . ")";
                $where .= " AND cat.parent_id = '" . $formvars['parent_id'] . "'";
            } else {
                $where .= " AND pc.categorie_id = '" . $formvars['categorie_id'] . "' ";
            }
        }

        // @TODO : voir pour multicategorie
        if (!empty($formvars['categorie_ext_id'])) {
            if (empty($formvars['categorie_id'])) {
                $sql .= " LEFT JOIN produit_categorie pc ON (p.produit_id = pc.produit_id)";
            }
            $sql .= " LEFT JOIN categorie cat ON ( pc.categorie_id = cat.categorie_id)";
            $where .= " AND FIND_IN_SET(" . $formvars['categorie_ext_id'] . ",cat.categorie_ext_id)";
        }

        if (!empty($formvars['gamme'])) {
            if (is_array($formvars['gamme'])) {
                $formvars['gamme'] = implode(',', $formvars['gamme']);
            }
            $sql .= " LEFT JOIN gamme_nom gn ON(gn.gamme_id = pg.gamme_id)";
            $where .= " AND pg.gamme_id IN (" . $formvars['gamme'] . ")";
        }

        if (!empty($formvars['caracteristique'])) {
            $caracteristique = new Caracteristique;
            $listCaraSort= $caracteristique->parseSql($formvars['caracteristique']);
            $j=1;
            foreach ($listCaraSort as $cara) {
                $sql.= " LEFT JOIN produit_caracteristique_valeur pcv".$j." ON (p.produit_id = pcv".$j.".produit_id)";
                $where.=" AND ";
                $where.=" pcv".$j.".caracteristique_valeur_id IN  (" . implode(',', $cara);
                $where.= ")";
                $j++;
            }
        }

        $sql.=" WHERE 1";
        // on ne remonte que les produits avec au moins un attribut actif
        $sql.=" AND patt.produit_attribut_id IS NOT NULL";
        if (!empty($formvars['statut'])) {
            $sql.=" AND p.produit_statut_id IN (".implode(',', $formvars['statut']).")";
            $sql.=" AND pb.produit_boutique_statut_id IN (".implode(',', $formvars['statut']).")";
        } else {
            $sql.=" AND p.produit_statut_id = 1";
            $sql.=" AND pb.produit_boutique_statut_id = 1";
        }

        if (!empty($formvars['marque'])) {
            if (is_array($formvars['marque'])) {
                $formvars['marque'] = implode(',', $formvars['marque']);
            }
            $sql .= " AND p.marque_id IN (" . $formvars['marque'] . ")";
        }

        if (!empty($formvars['produit_id_not_id'])) {
            $sql .= " AND p.produit_id NOT IN (" . $formvars['produit_id_not_id'] . ")";
        }

        if (!empty($formvars['motclef']) && mb_strlen($formvars['motclef']) < 3) {
            return array();
        }

        if (!empty($formvars['motclef'])) {

            // on nettoie la chaine
            $requete = trim(stripslashes($formvars['motclef']));
            // Recuperation des expressions
            preg_match_all('/"([^\"]+)"/', $requete, $matches);
            $expressions = $matches[1];
            // Recuperation des mots cles
            $requete = str_replace($expressions, "", $requete);
            $requete = str_replace('"', "", $requete);
            $motcles = explode(" ", $requete);

            foreach ($expressions as $value) {
                if (!empty($value)) {
                    $value = $this->sql->db->real_escape_string($value);
                    $sql.=" AND (";
                    $sql.=" pn.produit_nom_nom LIKE '%" . ($value) . "%'";
                    $sql.=" OR p.produit_ref_frs LIKE '%" . ($value) . "%'";
                    $sql.=" OR p.produit_ref LIKE '%" . ($value) . "%'";
                    $sql.=" OR m.marque_nom LIKE '%" . ($value) . "%'";
                    $sql.=" OR patt.produit_attribut_ref_frs LIKE '%" . ($value) . "%'";
                    $sql.=")";
                }
            }
            foreach ($motcles as $value) {
                if (!empty($value)) {
                    $value = $this->sql->db->real_escape_string($value);
                    $sql.=" AND (";
                    $sql.=" pn.produit_nom_nom LIKE '%" . ($value) . "%'";
                    $sql.=" OR p.produit_ref_frs LIKE '%" . ($value) . "%'";
                    $sql.=" OR p.produit_ref LIKE '%" . ($value) . "%'";
                    $sql.=" OR m.marque_nom LIKE '%" . ($value) . "%'";
                    $sql.=" OR patt.produit_attribut_ref_frs LIKE '%" . ($value) . "%'";
                    $sql.=")";
                }
            }
        }

        // search => pour la caisse
        if (!empty($formvars['search'])) {
            $formvars['search'] = $this->sql->db->real_escape_string($formvars['search']);
            $sql.=" AND pn.produit_nom_nom LIKE '%" . $formvars['search'] . "%'";
        }

        $sql .= $where;

        if (empty($formvars['group'])) {
            $formvars['group'] = '';
        }
        switch ($formvars['group']) {
            case 'attribut':
                $sql .= " GROUP BY patt.produit_attribut_id";
                break;
            case 'nom':
                $sql .= " GROUP BY pn.produit_nom_nom";
                break;
            default:
                $sql .= " GROUP BY p.produit_id";
                break;
        }
        // $sql.=" GROUP BY p.produit_id";
        $sql.=" HAVING 1";
        $sql.=" AND (prod_appro > 0 OR prod_qte > 0)";

        if (!empty($formvars['promo'])) {
            $sql.=" AND promo != ''";
        }
        if (!empty($formvars['not_in_spec'])) {
            $sql.=" AND p.produit_id NOT IN (".implode(",", $formvars['not_in_spec']).")";
        }

        if (!empty($formvars['prix'])) {
            if (!empty($formvars['prix'][0])) {
                $sql.=" AND produit_prix_ttc >= '" . $formvars['prix'][0] . "'";
            }
            if (!empty($formvars['prix'][1])) {
                $sql.=" AND produit_prix_ttc <= '" . $formvars['prix'][1] . "'";
            }
        }

        if (empty($formvars['order'])) {
            $formvars['order'] = '';
        }
        switch ($formvars['order']) {
            case 'tri-prix-croissant':
                $sql .= " ORDER BY produit_prix_ttc";
                break;
            case 'tri-prix-decroissant':
                $sql .= " ORDER BY produit_prix_ttc DESC";
                break;
            case 'tri-nom':
                $sql .= " ORDER BY produit_nom_nom";
                break;
            case 'random':
                $sql .= " ORDER BY RAND()";
                break;
        }

        // PAGINATION

        if (defined('PRODUIT_SUIV_PREC')) {

            // Gestion de la limit
            if (!empty($formvars['limit'])) {

                if (!empty($formvars['page']) && $formvars['page'] != "all") {
                    $offset = ($formvars['page'] - 1) * $formvars['limit'];
                    $sql.=" LIMIT " . $offset . ", " . $formvars['limit'];
                } elseif (empty($formvars['page'])) {
                    $sql.=" LIMIT " . $formvars['limit'];
                }
            }

            $arrData = $this->sql->getAll($sql);
        } else {
            if (!empty($formvars['debug']) && getenv('APP_DEBUG')) {
                dump($sql);
            }
            $arrData = $this->sql->getAll($sql);

            // Gestion de la limit
            if (!empty($formvars['limit'])) {

                $total = count($arrData);

                if (!empty($formvars['page']) && $formvars['page'] != "all") {
                    $offset = ($formvars['page'] - 1) * $formvars['limit'];
                    $arrData = array_slice($arrData, $offset, $formvars['limit']);
                } elseif (empty($formvars['page'])) {
                    $arrData = array_slice($arrData, 0, $formvars['limit']);
                }
            }
        }

        // PAGINATION
        if (empty($formvars['noBloc'])) {
            $arrProduit = array();
            foreach ($arrData as $p) {
                list($pTemp) = $this->get([
                    'produit_id' => $p['produit_id'],
                    'produit_attribut_id' => $p['produit_attribut_id']
                ]);
                $arrProduit[] = $pTemp;
            }

            return $arrProduit;
        } else {
            return $arrData;
        }
    }


    /**
     * @param array $formvars
     * @return array
     */
    public function getCategorie($formvars = array())
    {
        $sql = " SELECT DISTINCT pg.categorie_id";
        $sql .= " FROM produit_categorie pg";
        if (!empty($formvars['produit_id'])) {
            $sql .= " WHERE pg.produit_id = '" . $formvars['produit_id'] . "'";
        }
        if (!empty($formvars['produit_id_in'])) {
            $sql .=" WHERE pg.produit_id IN (" . $formvars['produit_id_in'] . ")";
        }
        $sql .= " GROUP BY pg.categorie_id";

        $gam = $this->sql->getAll($sql);

        $p_gam = array();
        $categorie = new Categorie;
        foreach ($gam as $g) {
            $tmpCateg = $categorie->get(array('categorie_id' => $g['categorie_id']));
            if (!empty($tmpCateg)) {
                if (!empty($tmpCateg[0]['parent_id'])) {
                    $tmpCateg[0]['grand_parent_nom'] = $categorie->get(
                        ['categorie_id' => $tmpCateg[0]['parent_id']],
                        'parent_nom_nom'
                    );
                }
                $p_gam[] = $tmpCateg[0];
            }
        }
        return $p_gam;
    }

    /**
     * @param array $formvars
     * @return array
     */
    public function getAssoc($formvars = array())
    {

        $sql = "SELECT pa.produit_a_id";
        $sql.=" FROM produit_assoc pa";
        $sql.=" LEFT JOIN produit p ON(p.produit_id = pa.produit_a_id)";
        $sql.=" WHERE 1";
        $sql.=" AND p.produit_statut_id = 1";

        if (!empty($formvars['produit_id'])) {
            $sql .= " AND pa.produit_id = '" . $formvars['produit_id'] . "'";
        }

        if (empty($formvars['order'])) {
            $formvars['order'] = '';
        }

        switch ($formvars['order']) {
            default:
                $sql .= " ORDER BY RAND()";
                break;
        }

        // Gestion de la limit
        if (!empty($formvars['limit'])) {
            $sql .= " LIMIT " . $formvars['limit'];
        }

        $arrProd = $this->sql->getAll($sql);

        $arrProduit = array();
        foreach ($arrProd as $p) {
            list($pTemp) = $this->get(array('produit_id' => $p['produit_a_id']));
            $arrProduit[] = $pTemp;
        }

        return $arrProduit;
    }

    /**
     * @param array $formvars
     * @param string $champs
     * @return array
     */
    public function getImage($formvars = array(), $champs = "")
    {

        $sql = "SELECT *";
        $sql.=" FROM produit_image";
        $sql.=" WHERE 1";
        if (!empty($formvars['produit_id'])) {
            $sql .= " AND produit_id = '" . $formvars['produit_id'] . "'";
        }
        if (isset($formvars['produit_image_default'])) {
            $sql .= " AND produit_image_default = '" . $formvars['produit_image_default'] . "'";
        }

        $sql.=" ORDER BY produit_image_rang";

        if (empty($champs)) {
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }


    /**
     * @param $produitId
     * @return bool
     */
    public function withAtt($produitId)
    {
        $sql = "SELECT COUNT(produit_attribut_id) as nb_att FROM (produit_attribut) WHERE produit_id='" . $produitId . "'";
        $result = $this->sql->getOne($sql);
        return ($result['nb_att'] > 0);
    }

    /**
     * @param $produitAttId
     * @return mixed
     */
    public function getIdProduit($produitAttId)
    {
        $sql = "SELECT produit_id FROM (produit_attribut) WHERE produit_attribut_id='" . $produitAttId . "'";
        $result = $this->sql->getOne($sql);
        return $result['produit_id'];
    }

    /**
     * @param array $formvars
     * @return array|null
     */
    public function getAutoComplete($formvars = array())
    {

        if (empty($formvars['id']) && mb_strlen($formvars['search']) < 2) {
            return array();
        }


        $sql = "SELECT p.produit_id, pn.produit_nom_nom, p.produit_ref, p.produit_ref_frs";
        $sql .= " FROM (produit p, produit_nom pn)";
        $sql.=" WHERE p.produit_id = pn.produit_id";
        $sql.=" AND pn.langue_id = '" . $_SESSION['langue_id'] . "'";
        $sql.=" AND p.produit_statut_id != 3";
        if (!empty($formvars['id'])) {
            $sql .= " AND p.produit_id = '" . $formvars['id'] . "'";
        }
        if (!empty($formvars['search'])) {
            $sql.=" AND (
                pn.produit_nom_nom LIKE '%" . $formvars['search'] . "%'
                OR p.produit_ref LIKE '%" . $formvars['search'] . "%'
                OR p.produit_ref_frs LIKE '%" . $formvars['search'] . "%'
                OR p.produit_id LIKE '%" . $formvars['search'] . "%'
            )";
        }
        return $this->sql->getAll($sql);
    }

    /**
     * @param array $formvars
     * @return array
     */
    public function getOption($formvars = array())
    {

        $lgId = Boutique::getInstance()->getLangue()->id;

        $sql = "SELECT distinct o.produit_option_valeur_id, povn.produit_option_valeur_nom, pon.produit_option_nom";
        if (!empty($formvars['with_cara'])) {
            $sql.=", cv.caracteristique_valeur_couleur";
        }
        $sql.=" FROM (produit_attribut a, produit_option_valeur o, produit_option po)";

        $sql.=" LEFT JOIN produit_option_valeur_nom povn ON (o.produit_option_valeur_id = povn.produit_option_valeur_id AND povn.langue_id='" . $lgId . "')";
        $sql.=" LEFT JOIN produit_option_nom pon ON (po.produit_option_id = pon.produit_option_id AND pon.langue_id='" . $lgId . "')";

        if (!empty($formvars['with_cara'])) {
            $sql .= " LEFT JOIN produit_option_valeur_caracteristique_valeur povcv ON(o.produit_option_valeur_id = povcv.produit_option_valeur_id)";
            $sql .= " LEFT JOIN caracteristique_valeur cv ON(povcv.caracteristique_valeur_id = cv.caracteristique_valeur_id)";
        }

        $sql.=" WHERE 1";

        if (!empty($formvars['produit_option_id'])) {
            if ($formvars['produit_option_id'] == self::$idTaille) {
                $sql .= " AND a.produit_option_valeur_id = o.produit_option_valeur_id";
            } elseif ($formvars['produit_option_id'] == self::$idCouleur) {
                $sql .= " AND a.produit_option_valeur_id_bis = o.produit_option_valeur_id";
            }

            $sql.=" AND po.produit_option_id = " . $formvars['produit_option_id'];
        }

        $sql.=" AND po.produit_option_id = o.produit_option_id";

        if (!empty($formvars['produit_option_nom'])) {
            $sql.=" AND pon.produit_option_nom = '".$formvars['produit_option_nom']."'";
        }
        if (empty($formvars['all'])) {
            $sql .= " AND produit_attribut_statut_id = '1'";
        }

        if (!empty($formvars['produit_id'])) {
            $sql.=" AND a.produit_id = '" . $formvars['produit_id'] . "'";
        }
        if (!empty($formvars['produit_attribut_id'])) {
            $sql .= " AND a.produit_attribut_id = '" . $formvars['produit_attribut_id'] . "'";
        }

        if (!empty($formvars['produit_option_valeur_id'])) {
            $sql .= " AND o.produit_option_valeur_id = '" . $formvars['produit_option_valeur_id'] . "'";
        }

        $sql.=" GROUP BY a.produit_attribut_id";

        $sql.=" ORDER BY o.produit_option_rang, povn.produit_option_valeur_nom";

        if (empty($champs)) {
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }

    /**
     * @deprecated
     *
     * @param $produitId
     * @param string $categorieId
     * @return array
     */
    public function getPrecSuiv($produitId, $categorieId = "")
    {
        $key = null;
        if (empty($_SESSION['listProduit'])) {
            $_SESSION['listProduit'] = array();
        }
        if ((empty($_SESSION['listProduit']) || ($key = array_search($produitId, $_SESSION['listProduit'])) === false) && !empty($categorieId)) {
            $this->getSearch(array('categorie_id' => $categorieId, 'noBloc' => true));
            $key = array_search($produitId, $_SESSION['listProduit']);
        }
        if ($key !== false) {

            $idPrec = $_SESSION['listProduit'][$key - 1];
            $idSuiv = $_SESSION['listProduit'][$key + 1];

            if (!empty($idPrec)) {
                $arrProd = $this->get(array('produit_id' => $idPrec));
                if (!empty($arrProd)) {
                    $urlPrec = $this->getUrl($arrProd[0]);
                }
            }
            if (!empty($idSuiv)) {
                $arrProd = $this->get(array('produit_id' => $idSuiv));
                if (!empty($arrProd)) {
                    $urlSuiv = $this->getUrl($arrProd[0]);
                }
            }

            $arrUrl = array('precedent' => $urlPrec, 'suivant' => $urlSuiv);

            return $arrUrl;
        }
    }

    /**
     * @param $arrProd
     * @return bool
     */
    public function getUrl($arrProd)
    {
        // TODO : revoir avec le routeur
        return false;
        // return Link::get("produit", array('produit_nom' => $arrProd['produit_nom_nom'], "marque_nom" => $arrProd['marque_nom'], "id" => $arrProd['produit_id']));
    }

    /**
     * @param null $pId
     * @return bool
     * @throws DediException
     */
    public function updateCaracteristique($pId = null)
    {
        // TODO : mettre a jour certaines cara en fonction des infos du produit
        if (is_null($pId) && empty($this->produit_id)) {
            throw new DediException('no id for updateCaracteristique()');
        }

        $p = (is_null($pId)) ? $this->properties : $this->getOne(array('p.produit_id' => $pId));

        if (empty($p)) {
            return false;
        }

        $caraValue = new CaracteristiqueValeur();
        $mask = require SITE_DIR . 'config/cara.php';

        foreach ($mask as $m) {
            $data = array();
            $param = array(
                'cv.caracteristique_id' => $m['caracteristique_id'],
            );
            if (!empty($m['code'])) {
                $param['cv.caracteristique_valeur_code'] = $p[$m['code']];
            } else {
                $param['cvn.caracteristique_valeur_nom_nom'] = $p[$m['field']];
            }
            $cv = $caraValue->get($param);

            if (empty($cv)) {
                $data['caracteristique_id'] = $m['caracteristique_id'];
                $data['caracteristique_valeur_statut_id'] = 1;
                $data['caracteristique_valeur_visible_filtre'] = 1;
                if (!empty($m['code'])) {
                    $data['caracteristique_valeur_code'] = $p[$m['code']];
                }
                $data['caracteristique_valeur_nom_nom'] = $p[$m['field']];
                $data['langue_id'] = Langue::DEFAULT_LANGUE_ID;
                $data['caracteristique_valeur_id'] = $this->sql->save('caracteristique_valeur', $data);
                $this->sql->save('caracteristique_valeur_nom', $data);
            } else {
                $data['caracteristique_valeur_id'] = $cv[0]['caracteristique_valeur_id'];
            }

            $this->sql->save('produit_caracteristique_valeur', array(
                'caracteristique_valeur_id' => $data['caracteristique_valeur_id'],
                'produit_id' => $p['produit_id'],
            ));
        }
    }
}
