<?php

/**
 * Class SiteSmarty
 */
class SiteSmarty extends Smarty
{
    public $template = null;
    public $cacheId = null;

    /**
     * contrcutor
     */
    public function __construct()
    {
        parent::__construct();

        // recup boutique
        $boutique = Boutique::getInstance();

        if (getenv('APP_MOBILE')) {
            $request = Application::getCurrentRequest();
            if ($request instanceof \Symfony\Component\HttpFoundation\Request && $request->cookies->has('version')) {
                $version = $request->cookies->get('version');
                $deviceDir = ($version == 'desktop') ? 'templates' : 'mobile';
            } else {
                $deviceDir = (Application::getMobileDetect()->isMobile() && !Application::getMobileDetect()->isTablet())
                    ? 'mobile' : 'templates';
            }
        } else {
            $deviceDir = 'templates';
        }

        $this->setTemplateDir(SITE_DIR . getenv('SMARTY_TEMPLATE_DIR') . DS . $boutique->dossier . DS . $deviceDir);
        $this->setCompileDir(SITE_DIR . getenv('SMARTY_COMPILE_DIR') . DS . $boutique->dossier);
        $this->setCacheDir(SITE_DIR . getenv('SMARTY_CACHE_DIR') . DS . $boutique->dossier);

        if (getenv('APP_DEBUG')) {
            $this->debugging = false;
            $this->compile_check = true;
        } else {
            $this->debugging = false;
            $this->compile_check = false;
        }

        if (getenv('APP_DEBUG') && getenv('SMARTY_DEBUG')) {
            $this->debugging = true;     // pour afficher la popup de debug
        }

        $this->addPluginsDir(__DIR__ . '/../../src/Smarty');
    }

    /**
     * @return null|string
     */
    public function activeCache()
    {
        if (getenv('APP_CACHE') && getenv('SMARTY_CACHE')) {
            $this->setCaching(Smarty::CACHING_LIFETIME_CURRENT);
            $this->setCacheLifetime(3600);
        }

        return $this->getCacheId();
    }

    /**
     * @return null|string
     */
    public function getCacheId()
    {
        if (is_null($this->cacheId)) {
            $cacheId = '';

            // tableau contenant les index que l'on doit inclure dans la construction de l'id du cache
            // on exclue par exemple tout les param GET de tracking (referer, utm, etc...)
            $paramGetInCacheId = array('tri', 'page', 'motclef');

            $arrCacheId = explode('?', $_SERVER['REQUEST_URI']);

            if (!empty($arrCacheId[1])) {
                $cacheId = $arrCacheId[0];
                // parse les param get sous forme de chaine pour reformer le tabelau $_GET
                parse_str($arrCacheId[1], $output);
                $get = array();
                foreach ($output as $k => $v) {
                    if (in_array($k, $paramGetInCacheId)) {
                        $get[$k] = $v;
                    }
                }

                // reconstruit le tableau $_GET en string
                if (!empty($get)) {
                    $cacheId .= '?' . http_build_query($get);
                }
            } else {
                $cacheId = $arrCacheId[0];
            }

            $this->cacheId = $cacheId;
        }

        return $this->cacheId;
    }

    public function isCached($template = null, $cache_id = null, $compile_id = null, $parent = null)
    {

        if (!is_null($template)) {
            $this->template = $template;
        }

        if (!is_null($this->template)) {
            if (!is_null($cache_id)) {
                $this->cacheId = $cache_id;
            } elseif (is_null($this->cacheId)) {
                $this->getCacheId();
            }

            return parent::isCached($this->template, $this->cacheId, $compile_id, $parent);
        } else {
            return false;
        }
    }


    public function display($template = null, $cacheId = null, $compileId = null, $parent = null, $display = false, $merge_tpl_vars = true, $no_output_filter = false)
    {

        if (!is_null($template)) {
            $this->template = $template;
        }
        if (!is_null($cacheId)) {
            $this->cacheId = $cacheId;
        }

        return parent::fetch($this->template, $this->cacheId, $compileId, $parent, $display, $merge_tpl_vars, $no_output_filter);
    }
    
    public function reassign($key, $value)
    {
        $curValue = $this->getTemplateVars($key);
        $this->assign($key, $curValue . $value);
    }
}
